/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugs.dialogs;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * 
 */
public abstract class StatusDialog extends Dialog {
    
    public final static int DEFAULT_WIDTH_IN_CHARS= 80;
    
    private final int fHeightInChars;
    private final int fWidthInChars;
    
    private final int fWidthOffset;
    private final int fHeightOffset;

    private final Map fErrorMessages;

    protected Button fOKButton;
    private Label fMessageLabel;
    private final String fTitle;
    
    private Composite fComposite;
    private boolean fComplete;
    
    protected StatusDialog(Shell parentShell, String title, int widthInChars, int heightInChars) {
        this(parentShell, title, widthInChars, heightInChars, 0, 0);
    }
        
    protected StatusDialog(Shell parentShell, String title, int widthInChars, int heightInChars, int widthOffsetInChars, int heightOffsetInChars) {
        super(parentShell);
        fWidthInChars= widthInChars;
        fHeightInChars= heightInChars;
        fWidthOffset= widthOffsetInChars;
        fHeightOffset= heightOffsetInChars;
        fTitle= title;
        setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.APPLICATION_MODAL);
        fErrorMessages= new HashMap();
    }

    protected StatusDialog(Shell parentShell, String title) {
    	this(parentShell, title, DEFAULT_WIDTH_IN_CHARS, SWT.DEFAULT);
    }
    
    protected Point getInitialSize() {
        final int width= fWidthInChars != SWT.DEFAULT ? convertWidthInCharsToPixels(fWidthInChars): SWT.DEFAULT;
        final int height= fHeightInChars != SWT.DEFAULT ? convertHeightInCharsToPixels(fHeightInChars): SWT.DEFAULT;
        final Point size= getShell().computeSize(width, height);
        if (fWidthOffset != SWT.DEFAULT)
            size.x += convertWidthInCharsToPixels(fWidthOffset);
        if (fHeightOffset != SWT.DEFAULT)
            size.y += convertHeightInCharsToPixels(fHeightOffset);
        return size;
    }
    
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(fTitle);
    }
    
    protected void createButtonsForButtonBar(Composite parent) {
        fOKButton = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
        fOKButton.setEnabled(fComplete);
    }
    
    protected final Control createDialogArea(Composite parent) {
        
        fComposite= (Composite)super.createDialogArea(parent);
        fComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        fComposite.setLayout(new GridLayout(1, false));
         
        final Composite mainComposite= createMainDialogArea(fComposite);
        mainComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        
		fMessageLabel = new Label(fComposite, SWT.WRAP);
		
		final GridData gridData= new GridData(SWT.FILL, SWT.BEGINNING, true, false);
		gridData.widthHint= 0;
		
		fMessageLabel.setLayoutData(gridData);
		fMessageLabel.setForeground(getShell().getDisplay().getSystemColor(SWT.COLOR_RED));

        applyDialogFont(fComposite);
        
        return fComposite;
    }
    
	protected void setComplete(boolean complete) {
	    fComplete= complete;
	    if (fOKButton != null) {
	        fOKButton.setEnabled(fComplete);
	    }
	}
	
	protected void setErrorMessage(Object key, String message) {

	    final boolean error= message != null;
	    if (error) {
	        fErrorMessages.put(key, message);
	        fMessageLabel.setText(message);
	        setComplete(false);
	    } else {
	        fErrorMessages.remove(key);
	        final boolean hasOtherErrors= fErrorMessages.size() > 0;
	        fMessageLabel.setText(hasOtherErrors ? getNextMessage() : "");
	        setComplete(!hasOtherErrors);
	    }
	}
	
	private String getNextMessage() {
        if (fErrorMessages.size() == 0)
            return null;
        final Entry entry= (Entry)fErrorMessages.entrySet().iterator().next();
        return (String)entry.getValue();
	}
	
	abstract protected Composite createMainDialogArea(Composite parent);
}
