/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugs.internal.main;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.team.bugs.extension.BugProviderFactory;
import org.eclipse.team.bugs.extension.BugsException;
import org.eclipse.team.bugs.internal.registry.BugProviderFactoryDescription;
import org.eclipse.team.bugs.internal.registry.ExtensionManager;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.team.bugs.tools.Utils;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;

/**
 * 
 */
public class StoredProvidersManager {

    private static final String PLUGINS_NODE = "providerPlugins";

    private final ArrayList fPluginsInUse;
    
    /**
     * 
     */
    public StoredProvidersManager() {
        fPluginsInUse= new ArrayList();
    }
    
    public Map getStoredProviders() {
        final Map result= new HashMap();
        try {
            final List storedIDs= Arrays.asList(getPluginsNode().childrenNames());
            final BugProviderFactoryDescription [] descriptions= ExtensionManager.getInstance().getBugProviderFactoryDescriptions();
            for (int i = 0; i < descriptions.length; i++) {
                if (storedIDs.contains(descriptions[i].getId())) {
                    result.putAll(instantiateStoredProviders(descriptions[i]));
                } 
            }
        } catch (BackingStoreException e) {
            BugsPlugin.log(BugsException.asBugsException(e));
        }
        return result;
    }
    
    private Map instantiateStoredProviders(BugProviderFactoryDescription description) {
        final BugProviderFactory factory;
        try {
            factory= description.createFactory();
        } catch (CoreException e) {
            BugsPlugin.log(e);
            return Utils.EMPTY_MAP;
        }
        final IBugProvider [] providers= factory.getStoredProviders();
        final Map result= new HashMap();
        for (int i = 0; i < providers.length; i++) {
            result.put(providers[i], factory);
        }
        return result;
    }

    private static Preferences getPluginsNode() {
        return new InstanceScope().getNode(BugsPlugin.ID).node(PLUGINS_NODE);
    }
    
    public void registerPluginID(String id) {
        if (!fPluginsInUse.contains(id)) {
            try {
                final Preferences node= getPluginsNode().node(id); 
                node.put("ID", id);
                node.flush();
            } catch (BackingStoreException e) {
            }
        }
        fPluginsInUse.add(id);
    }

    public void unregisterPluginID(final String id) {
        fPluginsInUse.remove(id);
        if (!fPluginsInUse.contains(id)) {
            try {
                final Preferences pluginsNode= getPluginsNode();
                pluginsNode.node(id).removeNode();
                pluginsNode.flush();
            } catch (BackingStoreException e) {
            }
        }
    }
}
