/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugs.internal.views.dnd;

import java.util.AbstractCollection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.widgets.Control;
import org.eclipse.team.bugs.internal.views.BugView;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.team.bugs.model.IGenericModel;

/**
 * 
 */
public class DropManager implements DropTargetListener {
    
    /**
     * A helper class to store the listeners, along with their supported
     * transfers and operations.
     */
    private static class ListenerInfo {
        
        private DropTargetListener fListener;
        private int fOperations;
        private Transfer [] fTransfers;
        
        public DropTargetListener getListener() {
            return fListener;
        }
        
        public int getOperations() {
            return fOperations;
        }
        
        public Transfer [] getTransferTypes() {
            return fTransfers;
        }
        
        public boolean supports(TransferData [] transferData, int operations) {
            if ((fOperations & operations) == 0)
                return false;
            final List transfers= Arrays.asList(fTransfers);
            for (Iterator iter = transfers.iterator(); iter.hasNext();) {
                final Transfer transfer= (Transfer)iter.next();
                for (int i = 0; i < transferData.length; i++) {
                    if (transfer.isSupportedType(transferData[i])) {
                        return true;
                    }
                }
            }
            return false;
        }
        
        public ListenerInfo(DropTargetListener listener, int operations, Transfer [] transferTypes) {
            fListener= listener;
            fOperations= operations;
            fTransfers= transferTypes;
        }
    }
    
    private final Map fListenerInfos;
    private final AbstractCollection fTransfers;
    private final DropTarget fDropTarget;
    private Control fControl;
    
    /**
     * @param view
     */
    public DropManager(BugView view) {
        super();
        fListenerInfos= new HashMap();
        fTransfers= new ArrayList();
        fControl= view.getViewer().getControl();
        fDropTarget= new DropTarget(fControl, DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK | DND.DROP_DEFAULT | DND.DROP_TARGET_MOVE);
        fDropTarget.setTransfer(new Transfer[0]);
        fDropTarget.addDropListener(this);
    }

    /**
     * Register a <code>DropTargetListener</code> for the given provider. For every provider, one listener
     * can be registered. If this method is called and a listener for the given provider is already registered, 
     * the previous listener is removed and the new one is added.
     * @param provider the provider
     * @param operations the supported operations
     * @param transferTypes the supported types
     * @param listener the listener to be notified
     */
    public void addDropSupport(IBugProvider provider, int operations, Transfer[] transferTypes, DropTargetListener listener) {
	    if (fListenerInfos.containsKey(provider))
	        internalRemoveSupport(provider);
        fListenerInfos.put(provider, new ListenerInfo(listener, operations, transferTypes));
        fTransfers.addAll(Arrays.asList(transferTypes));
        updateDropTarget();
	}
    
    /**
     * Remove the listener for the given provider.
     * @param provider the provider.
     */
    public void removeDropSupport(IBugProvider provider) {
        internalRemoveSupport(provider);
        updateDropTarget();
    }
    
    /**
     * Update the drag source with the set of all supported transfer types.
     */
    private void updateDropTarget() {
        final Set set= new HashSet(fTransfers);
        final Transfer [] transfers= new Transfer[set.size()];
        set.toArray(transfers);
        fDropTarget.setTransfer(transfers);
    }

    /**
     * A helper method to unregister the listener for a provider. 
     * @param provider the provider
     */
    private void internalRemoveSupport(IBugProvider provider) {
        final ListenerInfo info= (ListenerInfo)fListenerInfos.remove(provider);
        if (info == null)
            return;
        final Transfer [] transfers= info.getTransferTypes();
        for (int i = 0; i < transfers.length; i++) {
            fTransfers.remove(transfers[i]);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragEnter(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragEnter(DropTargetEvent event) {
        final ListenerInfo info= getCurrentInfo(event);
        if (info != null && info.supports(event.dataTypes, event.operations)) {
            info.getListener().dragEnter(event);
            return;
        }
        event.detail= DND.DROP_NONE;
        event.currentDataType= null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragLeave(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragLeave(DropTargetEvent event) {
        final ListenerInfo info= getCurrentInfo(event);
        if (info != null && info.supports(event.dataTypes, event.operations)) {
            info.getListener().dragLeave(event);
            return;
        }
        event.detail= DND.DROP_NONE;
        event.currentDataType= null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragOperationChanged(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragOperationChanged(DropTargetEvent event) {
        final ListenerInfo info= getCurrentInfo(event);
        if (info != null && info.supports(event.dataTypes, event.operations)) {
            info.getListener().dragOperationChanged(event);
            return;
        }
        event.detail= DND.DROP_NONE;
        event.currentDataType= null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragOver(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragOver(DropTargetEvent event) {
        final ListenerInfo info= getCurrentInfo(event);
        if (info != null && info.supports(event.dataTypes, event.operations)) {
            info.getListener().dragOver(event);
            return;
        } 
        event.detail= DND.DROP_NONE;
        event.currentDataType= null;
    }


    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#drop(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void drop(DropTargetEvent event) {
        final ListenerInfo info= getCurrentInfo(event);
        if (info.supports(event.dataTypes, event.operations)) {
            info.getListener().drop(event);
        } else {
            event.detail= DND.DROP_NONE;
            event.currentDataType= null;
        }
    }


    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dropAccept(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dropAccept(DropTargetEvent event) {
        final ListenerInfo info= getCurrentInfo(event);
        if (info.supports(event.dataTypes, event.operations)) {
            info.getListener().dropAccept(event);
        } else {
            event.detail= DND.DROP_NONE;
            event.currentDataType= null;
        }
    }
    
    protected ListenerInfo getCurrentInfo(DropTargetEvent event) {
        if (event.item == null)
            return null;
        final IGenericModel element= (IGenericModel)event.item.getData();
        if (element == null)
            return null;
        final IBugProvider provider= element.getProvider();
        return (ListenerInfo)fListenerInfos.get(provider);
    }
}
