/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.actions;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.dialogs.UserValidationDialog;
import org.eclipse.team.bugzilla.model.BugzillaProvider;
import org.eclipse.team.bugzilla.operations.LoginOperation;

/**
 * Action which logs into a bugzilla server
 */
public class LoginAction extends BugzillaActionDelegate {

	public static final String INFO_PASSWORD = "org.eclipse.team.bugzilla.password";//$NON-NLS-1$ 
	public static final String INFO_USERNAME = "org.eclipse.team.bugzilla.username";//$NON-NLS-1$ 
    public static final String AUTH_SCHEME = "";//$NON-NLS-1$ 
    public static final String REALM = "bugs";//$NON-NLS-1$ 
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.actions.BugzillaActionDelegate#isEnabled()
     */
    protected boolean isEnabled() throws CoreException {
        return getSelectedContainer() != null;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.ui.actions.ActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
        try {
            BugzillaProvider provider = (BugzillaProvider)getSelectedContainer();
            performLogin(getTargetPart().getSite().getShell(), provider, null);
        } catch (BugzillaException e) {
            ErrorDialog.openError(getTargetPart().getSite().getShell(), null, null, e.getStatus());
        }
    }

    /**
     * Log into the server associated with the given provider.
     * @param shell the shell to parent the login prompt
     * @param provider the provider
     * @param message the message to be displayed in the prompt
     * @return whether the login succeeded or was cancelled
     * @throws BugzillaException
     */
    public static boolean performLogin(Shell shell, BugzillaProvider provider, String message) throws BugzillaException {
        
        // Initialize state that is used to control prompting loop
        boolean keepTrying = true;
        BugzillaException exception = null;
        String username = "";
        String password = null;
        boolean allowCaching = false;
        
        // Look for cached authentication information
        Map authInfo = getCachedAuthentication(provider);
        if (authInfo != null) {
            username = (String)authInfo.get(INFO_USERNAME);
            password = (String)authInfo.get(INFO_PASSWORD);
            if (username != null && password != null) {
                try {
                    // Information was found so try and login
                    allowCaching = true;
                    new LoginOperation().run(provider, username, password);
                    
                    // We succeeded so there is no need to prompt
                    return true;
                } catch (BugzillaException e) {
    	            exception = e;
    	            keepTrying = true;
                }
            }
        }
        
        // Loop until we can login or the user cancels
        while (keepTrying) {
	        try {
	            keepTrying = false;
	            if (exception != null) {
	                message = exception.getMessage();
	            }
	            UserValidationDialog dialog = new UserValidationDialog(shell, provider.getConfiguration().getBaseURL(), username, message);
	            dialog.setUsername(username);
	            dialog.setPassword(password);
	            dialog.setAllowCaching(allowCaching);
	            if (dialog.open() == Window.OK) {      
	                username = dialog.getUsername();
                    password = dialog.getPassword();
                    new LoginOperation().run(provider, username, password);
                    allowCaching = dialog.getAllowCaching();
                    updateCache(provider, username, password, allowCaching);
	                return true;
	            }
	            exception = null;
	        } catch (BugzillaException e) {
	            exception = e;
	            keepTrying = true;
	        }
        }
        if (exception != null) {
            throw exception;
        }
        return false;
    }

    private static Map getCachedAuthentication(BugzillaProvider provider)  {
        String baseURL = provider.getConfiguration().getBaseURL();
		try {
            URL url = new URL(baseURL);
			Map map = Platform.getAuthorizationInfo(url, REALM, AUTH_SCHEME);
			if (map != null)
			    return map;
		} catch (MalformedURLException e) {
		    BugzillaPlugin.log("Invalid server URL: " + baseURL, e);
        }
		return null;
    }
    
    private static void updateCache(BugzillaProvider provider, String username, String password, boolean cachePassword) {
		// put the password into the Platform map
		String baseURL = provider.getConfiguration().getBaseURL();
		try {
            URL url = new URL(baseURL);
	        Map map = Platform.getAuthorizationInfo(url, REALM, AUTH_SCHEME);
			if (map == null) {
				map = new java.util.HashMap(10);
			}
			if (username != null)
				map.put(INFO_USERNAME, username);
			if (cachePassword && password != null)
				map.put(INFO_PASSWORD, password);
			Platform.addAuthorizationInfo(url, REALM, AUTH_SCHEME, map);
		} catch (CoreException e) {
			// Just log the exception
			BugzillaPlugin.log(e);
		} catch (MalformedURLException e) {
		    BugzillaPlugin.log("Invalid server URL: " + baseURL, e);
        }
    }

}
