/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.datatypes;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * An abstract data holder for key-value pairs where all the possible keys are
 * predefined. Data supports commits, where data items can be changed 
 */
public abstract class Data {
    
    private final static DateFormat sDateFormat= new SimpleDateFormat("yyyy-MM-dd HH:mm"); //$NON-NLS-1$
    
    /** Original values */
    private final Map fValues;
    
    /** Changed but uncommitted values */
    private final Map fChangedValues;

    /** Create an empty Data */
    public Data() {
        fValues= new HashMap();
        fChangedValues= new HashMap();
    }
    
    /** Create a Data with starting values defined by fValues
     * 
     * @param fValues the starting values for this <code>Data</code>
     */
    public Data (Map fValues)
    {
    	this.fValues = fValues;
    	this.fChangedValues = new HashMap();
    }

    /** Sets key to value without commiting this change. Use commitChanged()
     * or commitChanged(key) to commit this change.
     * 
     * @param key the key
     * @param value the new value
     */
    public void set(String key, String value) {
        if (!getKeys().contains(key))
            return;
        
        final String oldValue= (String)fValues.get(key);
        if (oldValue == null) {
            fValues.put(key, value);
            fChangedValues.remove(key);
        } else if (!oldValue.equals(value)) {
            fChangedValues.put(key, value);
        }
    }

    /** Sets key to value without commiting this change. Use commitChanged()
     * or commitChanged(key) to commit this change.
     * 
     * @param key the key
     * @param value the new value
     */
    public void set(String key, Date date) {
        set(key, sDateFormat.format(date));
    }
    
    /** Sets key to value without commiting this change. Use <code>commitChanged()</code>
     * or <code>commitChanged(String key)</code> to commit this change.
     * 
     * @param key the key
     * @param value the new value
     */
    public void set(String key, boolean state) {
        set(key, Boolean.valueOf(state).toString());
    }
    
    /**
     * Returns a <code>String</code> representation of the value for the given key, looking
     * in the changed values before looking at commited values.
     * 
     * @param key the key
     * @return <code>String</code> representation of the value corresponding to the
     * given key or  the empty String if the key is not set
     */
    public String getString(String key) {
        if (fChangedValues.containsKey(key))
            return (String)fChangedValues.get(key);
        if (!fValues.containsKey(key))
        	return "";
        Object d = fValues.get(key);
        return d.toString();
    }
    
    /**
     * Returns a <code>Date</code> representation of the value for the given key, 
     * looking in the changed values before looking at commited values.
     * 
     * @param key the key
     * @return the <code>Date</code> representation of the value corresponding to the
     * given key or null if it's either not a valid date or not set
     */
    public Date getDate(String key) {
        try {
            return sDateFormat.parse((String)fValues.get(key));
        } catch (ParseException e) {
            return null; // should not happen!
        }
    }
    
    /**
     * Returns a <code>boolean</code. representation of the value for the given key, 
     * looking in the changed values before looking at commited values.
     * 
     * @param key the key
     * @return the <code>boolean</code> representation of the value corresponding to
     * the given key
     */
    public boolean getBoolean(String key) {
        return Boolean.valueOf(getString(key)).booleanValue();
    }
    
    /**
     * Checks if the value of the given key has been changed, and is uncommitted
     * @param key the key
     * @return true if the value associated with the given key has been changed
     * but not yet committed, false otherwise
     */
    public boolean hasChanged(String key) {
        return fChangedValues.containsKey(key);        
    }
    
    /**
     * Checks if there is a value associated with the given key 
     * @param key they key
     * @return true if there is a value associated with the given key, false
     * otherwise
     */
    public boolean exists(String key) {
        return fValues.containsKey(key);
    }
    
    /**
     * Commits all changed values.
     */
    public void commitChanged() {
        fValues.putAll(fChangedValues);
    }
    
    /**
     * Commits the value corresponding to the given key if it has been changed,
     * has no effect if the value for this key does not exist or has not been
     * changed.
     * 
     * @param key the key
     */
    public void commitChanged(String key) {
        if (fChangedValues.containsKey(key)) {
            fValues.put(key, fChangedValues.get(key));
        }
    }
    
    /**
     * Clients must implement this to define the set of keys which are allowed. 
     * Iteration and lookup should be fast, so a LinkedHashSet may be a good choice. 
     * @return the set of allowed keys
     */
    protected abstract Collection getKeys();
}
