/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.datatypes;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

/**
 * Data about a single bug in a bugzilla database. Contains all the information
 * about the bug, including comments, offline. 
 * 
 * @author JavaDoc additions: Igor Foox <ifoox@redhat.com>
 */
public class ReportData extends Data {
    
    public final static String ID= "bug_id";
    public final static String STATUS= "bug_status";
    public final static String PRODUCT= "product";
    public final static String PRIORITY= "priority";
    public final static String VERSION= "version";
    public final static String PLATFORM= "rep_platform";
    public final static String ASSIGNEDTO= "assigned_to";
    public final static String DELTATIMESTAMP= "delta_ts";
    public final static String COMPONENT= "component";
    public final static String TARGETMILESTONE= "target_milestone";
    public final static String SEVERITY= "bug_severity";
    public final static String QACONTACT= "qa_contact";
    public final static String OPERATINGSYSTEM= "op_sys";
    public final static String RESOLUTION= "resolution";
    public final static String BUGFILELOCATION= "bug_file_loc";
    public final static String STATUSWHITEBOARD= "status_whiteboard";
    public final static String TITLE= "short_desc";
    public final static String BUGZILLAVERSION= "bz_version";
    public final static String LONGDESCS= "longdescs";
    public final static String CC= "cc";
    public final static String BLOCKS= "blocked";
    public final static String DEPENDSON= "dependson";
    
    /** Set of possible keys */
    private final static LinkedHashSet sSet;
    
    static {
        sSet= new LinkedHashSet();
        sSet.add(BUGZILLAVERSION);
        sSet.add(ID);
        sSet.add(STATUS);
        sSet.add(PRODUCT);
        sSet.add(PRIORITY);
        sSet.add(VERSION);
        sSet.add(PLATFORM);
        sSet.add(ASSIGNEDTO);
        sSet.add(DELTATIMESTAMP);
        sSet.add(COMPONENT);
        sSet.add(TARGETMILESTONE);
        sSet.add(SEVERITY);
        sSet.add(QACONTACT);
        sSet.add(OPERATINGSYSTEM);
        sSet.add(RESOLUTION);
        sSet.add(BUGFILELOCATION);
        sSet.add(STATUSWHITEBOARD);
        sSet.add(TITLE);
    }
    
    /* Additional values */ 
    public List keywords;
    public List dependsOn;
    public List blocks;
    public List ccs;
    public List comments;
    
    /** Create a new, empty, ReportData */
    public ReportData() {
        super();
        keywords= new ArrayList();
        dependsOn= new ArrayList();
        blocks= new ArrayList();
        ccs= new ArrayList();
        comments= new ArrayList();
    }
    
    /** Create a new ReportData with initial values filled in from m
     * 
     * @param m initial set of key-value pairs
     */
    public ReportData(Map m) {
    	super(m);
        keywords= new ArrayList();
        dependsOn= new ArrayList();
        blocks= new ArrayList();
        ccs= new ArrayList();
        comments= new ArrayList();
    }
    
    

    /** 
     * String representation of this Report.
     */
    public String toString() {
        
        final StringBuffer buffer= new StringBuffer();
        final String nl= System.getProperty("line.separator"); //$NON-NLS-1$
        final String space= " "; //$NON-NLS-1$
        
        buffer.append("ID: " + getString(ID) + nl); //$NON-NLS-1$
        buffer.append("Status: " + getString(STATUS) + nl); //$NON-NLS-1$
        buffer.append("Product: " + getString(PRODUCT) + nl); //$NON-NLS-1$
        buffer.append("Priority: " + getString(PRIORITY) + nl); //$NON-NLS-1$
        buffer.append("Version: " + getString(VERSION) + nl); //$NON-NLS-1$
        buffer.append("Platform: " + getString(PLATFORM) + nl); //$NON-NLS-1$
        buffer.append("Assigned To: " + getString(ASSIGNEDTO) + nl); //$NON-NLS-1$
        buffer.append("Delta Time Stamp: " + getString(DELTATIMESTAMP) + nl); //$NON-NLS-1$
        buffer.append("Component: " + getString(COMPONENT) + nl); //$NON-NLS-1$
        buffer.append("Target Milestone: " + getString(TARGETMILESTONE) + nl); //$NON-NLS-1$
        buffer.append("Severity: " + getString(SEVERITY) + nl); //$NON-NLS-1$
        buffer.append("Operating System: " + getString(OPERATINGSYSTEM) + nl); //$NON-NLS-1$
        
        buffer.append("Keywords:"); //$NON-NLS-1$
        for (Iterator iter = keywords.iterator(); iter.hasNext();) {
            buffer.append(space + iter.next());
        }

        buffer.append(nl + "Depends On:"); //$NON-NLS-1$
        for (Iterator iter = dependsOn.iterator(); iter.hasNext();) {
            buffer.append(space + iter.next());
        }
        
        buffer.append(nl + "Blocks:"); //$NON-NLS-1$
        for (Iterator iter = blocks.iterator(); iter.hasNext(); ) {
            buffer.append(space + iter.next());
        }

        buffer.append(nl + "CCs:"); //$NON-NLS-1$
        for (Iterator iter = ccs.iterator(); iter.hasNext(); ) {
            buffer.append(space + iter.next());
        }
        
        buffer.append(nl + "Comments: "); //$NON-NLS-1$
        for (Iterator iter = comments.iterator(); iter.hasNext(); ) {
            buffer.append(space + iter.next());
        }
        return buffer.append(nl).toString();
    }

    /**
     * Returns the set of all possible keys
     */
    protected Collection getKeys() {
        return sSet;
    }
 }
