/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.team.bugzilla.BugzillaException;

/**
 * A generic model element container
 */
public abstract class BugzillaContainer extends BugzillaModel {

    private Map fChildren;
    
    public BugzillaContainer() {
        super();
        fChildren= new LinkedHashMap();
    }

    public BugzillaModel[] getChildren() {
        return (BugzillaModel[]) fChildren.values().toArray(new BugzillaModel[fChildren.size()]);
    }
    
    /**
     * Add a child to this container and set the child's path correctly. If this
     * container already contains a child with the same ID, the existing child
     * is removed and replaced by the new one. This method assumes that
     * <code>isReadOnly()</code> has been checked before, if applicable, and
     * thus inserts new children regardless of this property. A <code>childAdded</code> event
     * is fired after the child has been inserted.
     * 
     * @param children The children to be added.
     */
    public void addChildren(BugzillaModel [] children) {
        
        if (children.length == 0)
            return;
        
        final List existing= new ArrayList();
        for (int i = 0; i < children.length; i++) {
            final BugzillaModel child= children[i];
            if (contains(child)) {
                existing.add(fChildren.get(child.getID()));
            }
        }
        removeChildren((BugzillaModel [])existing.toArray(new BugzillaModel [existing.size()]));
        
        for (int i = 0; i < children.length; i++) {
            final BugzillaModel child= children[i];
            child.setParent(this);
            fChildren.put(child.getID(), child);
        }
        fireChildrenAdded(children);
    }
    
    public boolean contains(String id) {
        return fChildren.containsKey(id);
    }
    
    public boolean contains(BugzillaModel child) {
        return contains(child.getID()) && getChild(child.getID()) == child;
    }

    /**
     * Remove a child from this container. The child's parent is set to null
     * and a <code>childRemoved</code> event is fired.
     * @param children The childrem to be removed.
     */
    public void removeChildren(BugzillaModel [] children) {
        
        if (children.length == 0)
            return;
        
        final List removed= new ArrayList();
        for (int i = 0; i < children.length; i++) {
            final BugzillaModel child = children[i];
            if (!contains(child)) 
                continue;
            fChildren.remove(child.getID());
            child.setParent(null);
            removed.add(child);
        }
        fireChildrenRemoved((BugzillaModel [])removed.toArray(new BugzillaModel [removed.size()]));
    }
    
    public void setRefreshing(boolean refreshing) {
        if (isRefreshing() == refreshing) 
            return;
        super.setRefreshing(refreshing);
        setChildrenRefreshing(refreshing);
    }
    
    /**
     * @param refreshing
     */
    protected void setChildrenRefreshing(boolean refreshing) {
        for (final Iterator iter = fChildren.values().iterator(); iter.hasNext();) {
            ((BugzillaModel)iter.next()).setRefreshing(refreshing);
        }
    }

    public boolean hasParent(Object candiate) {
        
        final BugzillaContainer parent= (BugzillaContainer)getParent();
        if (parent == null)
            return false;
                
        return this == candiate || parent.hasParent(candiate);
    }
    
    public void accept(IBugzillaModelVisitor visitor) throws BugzillaException {
        if (visitor.visit(this)) {
            BugzillaModel[] children = getChildren();
            for (int i = 0; i < children.length; i++) {
                BugzillaModel model = children[i];
                model.accept(visitor);
            }
        }
    }

    /**
     * Get the name of the container as it would appear in any lists or trees.
     * This name is used to ensure a unique name within a parent container.
     * @return the name of the container
     */
    public abstract String getName();
  
    /**
     * Return the child with the given id or <code>null</code>
     * if one does not exist.
     * @param id the id of a child
     * @return the child with the given id or <code>null</code>
     */
    public BugzillaModel getChild(String id) {
        return (BugzillaModel)fChildren.get(id);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#resetPath()
     */
    protected void resetPath() {
        super.resetPath();
        for (Iterator iter = fChildren.values().iterator(); iter.hasNext();) {
            final BugzillaModel child = (BugzillaModel) iter.next();
            child.resetPath();
        }
    }
    
    /**
     * This method indicates whether a user should be able to add or delete any children
     * of this container.
     * @return True if a user can modify the children, false otherwise.
     */
    public abstract boolean canModifyChildren();
}
