/*
 * Created on 3-Dec-2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.eclipse.team.bugzilla.model;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.operations.HTTPConnection;

/**
 * A class to represent options for a bugzilla query. 
 * Products are mapped to an array of lists of available components, versions, and milestones.
 * 
 */
public class BugzillaQueryOptions implements IRemoteModel {
	
	/* Bugzilla option components (component, version, target-milestone)
	 * as defined by Bugzilla
	 */    
	public static final String COMPS[] = {"cpts", "vers", "tms"}; 
	public static final int COMPONENT_INDEX = 0;
	public static final int VERSION_INDEX = 1;
	public static final int TARGETMILESTONE_INDEX = 2;
	/* A set of all other desired query options to collect */
	public static final String BUG_STATUS = "bug_status";
	public static final String RESOLUTION = "resolution";
	public static final String SEVERITY = "bug_severity";
	public static final String PRIORITY = "priority";
	public static final String PLATFORM = "rep_platform";
	public static final String OPERATING_SYSTEM = "op_sys";
	
	private Map productComponentOptions; // maps products to an array of list's of components.
	private Map alternativeOptions;      // maps other options to a list of possible values.
	private List keywords;				 // a list of possible keywords.
	private String queryPageUrl;
	private String baseUrl;
	private boolean isRefreshing;
	private boolean needsRefresh;
	
	public BugzillaQueryOptions(String queryPage) {
		productComponentOptions = new HashMap();
		alternativeOptions = new HashMap();
		keywords = new LinkedList();
		queryPageUrl = queryPage;
		baseUrl = queryPage;
		needsRefresh = true;
		setRefreshing(false);
	}	
	
	/**
	 * @return The URL of the query page for the bugzilla database
	 */
	public String getUrl() {
		return queryPageUrl;
	}
	
	/**
	 * 
	 * @return The base url of the bugzilla database
	 */
	public String getBaseUrl() {
		return baseUrl;
	}
	
	/**
	 * Return an iterator over all option keys in. Use this to find out what
	 * options are supported by the associated Bugzilla. This should be used in
	 * conjunction with <code>getOptions</code> to find out what are the 
	 * possible values for each option returned by the <code>Iterator</code> that
	 * this method returns.This method should NOT be used to retrieve the product 
	 * list, use <code>productIterator()</code> instead.
	 *  
	 * @return
	 */
	public Iterator optionKeyIterator() {
		return alternativeOptions.keySet().iterator();
	}
	/**
	 * Return the list of options associated with the given value or <code>null</code> if
	 * none exists. This method should NOT be used to retrieve product options, use <code>
	 * getProductOptions(String product)</code> instead.
	 * 
	 * @param value The value of which the associated options will be returned
	 * @return The list of options associated with the given value or <code>null</code> if
	 * none exists.
	 */
	public List getOptions(String value) {
		return (List)alternativeOptions.get(value);
	}
	
	/**
	 * Set the list of options associated with the given value to the given 
	 * <code>List</code>. This method should NOT be used to set product options,
	 * instead the <code>addProduct(String name)</code> method can be used to
	 * add a new product, and then can be manipulated using 
	 * <code>getProductOptions(String)</code>.
	 * 
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the alternative options list.
	 *  
	 * @param value The vlaue with which options will be associated
	 * @param options The list of options to associate with the govem value
	 */
	protected void setOptions(String value, List options) {
		alternativeOptions.put(value, options);
	}
	
	/**
	 * Add an option to the options associated with the given value if the
	 * option is not already in the set. If the value does not yet exist
	 * in the set it is added. 
	 * This method should NOT be used to add a product , instead the
	 *  <code>addProduct(String name)</code> method should be used for that purpose.
	 * 
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the alternative options list.
	 * 
	 * @param value The vlaue with which the option will be associated
	 * @param option The option to associate with the given value
	 */
	protected void addOption(String value, String option) {
		List options = (List) alternativeOptions.get(value);
		if (options == null) {
			options = new LinkedList();
			alternativeOptions.put(value, options);
		}
		
		if (!options.contains(option)) {
			options.add(option);
		}
	}
	
	/**
	 * Remove an option from the options associated with the given value. If no
	 * such option exists this method has no effect. If there are no more options
	 * associated with the given value, the value is removed altogether. This 
	 * method should NOT be used to remove product options, instead the
	 *  <code>getProductOptions(String)</code> method should be used 
	 *  for that purpose.
	 * 
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the alternative options list.
	 * 
	 * @param value The value with from whos associated options the given 
	 * option will be removed
	 * @param option The option to remove
	 */
	protected void removeOption(String value, String option) {
		List options = (List) alternativeOptions.get(value);
		options.remove(option);
		
		if (options.size() == 0) {
			alternativeOptions.remove(value);
		}
	}
	
	/**
	 * Removes the given value and all of its associated options from the set
	 * of alternative options. If the value is not in the set, this method has 
	 * no effect. This 
	 * method should NOT be used to remove products, instead the
	 *  <code>removeProduct(String)</code> method should be used for that 
	 *  purpose.
	 *  
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the alternative options list.
	 * 
	 * @param value The value to be removed from the set of alternative options
	 */
	protected void unsetOptions(String value) {
		if (alternativeOptions.keySet().contains(value))
			alternativeOptions.remove(value);
	}
	
	/**
	 * Returns an array of <code>List</code>'s (one <code>List</code> for each bugzilla
	 * component) of options associated with the given product.
	 * 
	 * @param product The product who's options are to be returned.
	 * @return An array of <code>List</code>'s of options for the product.
	 */
	public List[] getProductOptions(String product) {
		return (List[])productComponentOptions.get(product);
	}
	
	/**
	 * Returns an iterator of keys (products) for the bugzilla options.
	 * 
	 * @return An iterator for the bugzilla options.
	 */
	public Iterator productIterator() {
		return productComponentOptions.keySet().iterator();
	}
	
	/**
	 * Add a new product definition to the query options. This method has
	 * no effect if the product already exists.
	 * 
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the product options list.
	 * 
	 * @param name The name of the product to add
	 */
	protected void addProduct(String name){
		if (productComponentOptions.containsKey(name))
				return;
		
		List[] opts = new List[COMPS.length];
		
		// Add an empty list for each of the main types of information
		// in a product
		for (int i = 0; i < opts.length; i++) {
			opts[i] = new LinkedList();
		}
		
		productComponentOptions.put(name, opts);
	}
	
	/**
	 * Remove a product definition from the query options. This method has no
	 * effect if the product is not defined.
	 *
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the product options list.
	 * 
	 * @param name The name of the product to remove
	 */
	protected void removeProduct(String name){
		productComponentOptions.remove(name);
	}
	
	
	/**
	 * Returns an iterator of the aquired keywords.
	 * 
	 * @return An iterator of keywords.
	 */
	public Iterator keywordsIterator() {
		return keywords.iterator();
	}
	
	/**
	 * Return an iterator for the options associated with the given value or null.
	 * 
	 * @param value The value of which an iterator (for the associated options) will be returned.
	 * @return An iterator for the options associated with the given value or null.
	 */
	public Iterator optionsIterator(String value) {
		List options = getOptions(value);
		return (options == null)? null : options.iterator();
	}
	
	/**
	 * Refreshes the query options by contacting the Bugzilla to which this 
	 * <code>BugzillaQueryOptions</code> is connected and re-parse the options
	 * supplied by that Bugzilla.
	 */
	public void refresh(IProgressMonitor monitor) throws BugzillaException {
		
		setRefreshing(true);
		BugzillaQueryOptionParser p = new BugzillaQueryOptionParser(queryPageUrl);
		p.parseOptions(monitor);
		setRefreshing(false);
		needsRefresh = false;
	}
	
	/**
	 * Does nothing. FIXME: Should it?
	 */
	public void update() {
		//System.out.println("BugzillaQueryOptions: update()");
		
	}
	
	/**
	 * Returns whether this <code>BugzillaQueryOptions</code> needs to be 
	 * refreshed. This is only true when a new <code>BugzillaQueryOptions</code>
	 * is created that has not yet been refreshed.
	 */
	public boolean needsRefresh() {
		return needsRefresh;
	}
	
	/**
	 * Does nothing atm. FIXME: Should it return something?
	 */
	public IBugProvider getProvider() {
		// TODO Auto-generated method stub
		return null;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.bugs.model.IGenericModel#setRefreshing(boolean)
	 */
	public void setRefreshing(boolean refreshing) {
		isRefreshing = refreshing;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.bugs.model.IGenericModel#isRefreshing()
	 */
	public boolean isRefreshing() {
		return isRefreshing;
	}
	
	
	/*
	 * A class to parse the possible query options from a query page.
	 * At the moment bugzilla version don't vary enough to break them up into
	 * seperate classes. This may be needed in the future.
	 */
	private class BugzillaQueryOptionParser {
		
		//private static final String URL_SUFFIX_QUERY = "query.cgi?format=advanced";
		private static final String URL_SUFFIX_QUERY = "query.cgi?format=advanced&rememberjs=1";
		private static final String URL_SUFFIX_KEYWORDS = "describekeywords.cgi?";
		
		public BugzillaQueryOptionParser(String queryPage) { 
			setUrl(queryPage); 
		}
		
		public void setUrl(String queryPage) { 
			queryPageUrl = queryPage;
			
			if ( queryPageUrl.endsWith(".cgi") || queryPageUrl.endsWith(".cgi?") ||
					queryPageUrl.endsWith(URL_SUFFIX_QUERY)) {
				return;
			}
			
			if ( !queryPageUrl.endsWith("/") ) {
				queryPageUrl += "/";
			}
			
			/* FIXME: jpound - horrible hack to make bugzilla.redhat.com/bugzilla-old work */
//			if (queryPageUrl.indexOf("bugzilla.redhat.com/bugzilla-old") > 0) {
//				queryPageUrl += "query.cgi?rememberjs=1&format=";
//			}
//			else {
				queryPageUrl += URL_SUFFIX_QUERY;
//			}
		}
		
		public void parseOptions(String queryPage, IProgressMonitor monitor) throws BugzillaException {
			setUrl(queryPage);
			parseOptions(monitor);
		}
		
		/*
		 * Parse the query options for this bugzilla database. 
		 */
		public void parseOptions(IProgressMonitor monitor) throws BugzillaException {
			monitor.subTask("Fetching query options from " + queryPageUrl.substring(0,queryPageUrl.indexOf("?")));

			try {
				//BufferedReader read = new BufferedReader(new InputStreamReader( (new HTTPConnection(url)).getInputStream(monitor) ));
				String queryPage = (new HTTPConnection(queryPageUrl)).getString(monitor);
				monitor.worked(5);
				StringTokenizer strtok = new StringTokenizer(queryPage, "\n");
				String line = "";
				List allOptions = new LinkedList();
				
				/* locate start of product/component table declarations */
				while ( (strtok.hasMoreTokens()) && (line.indexOf(BugzillaQueryOptions.COMPS[0]+"[0]") < 0) ) 
					line = strtok.nextToken();
				
				if (!strtok.hasMoreTokens()) {
					System.err.println("BugzillaQueryOptionParser: Could not retrieve options");
					return;
				}
				
				/* 
				 * parse all 'BugzillaQueryOptions.COMPS' from table
				 * Options are stored in a list until the product names can be parsed
				 */
				while ( line.indexOf("[") >= 0 ) { // while there are still items
					
					List[] currentComps = new LinkedList[COMPS.length];
					allOptions.add(currentComps);
					for (int i = 0; i < COMPS.length; i++) {
						currentComps[i] = new LinkedList();
					}
					
					int compIndex = 0;
					while ( compIndex == 0 || (line.indexOf(COMPS[0]) < 0) ) { // loop untill we hit a new component
						if (line.indexOf("[") < 0 || line.indexOf("[") == line.lastIndexOf("[")) {
                            line = strtok.nextToken();
                            break;
                        }
						
                        line = line.substring(line.lastIndexOf("[")+1,line.lastIndexOf("]"));
                        
						StringTokenizer tok = new StringTokenizer(line, ",");
						
						while ( /*(tok.nextToken() != null) &&*/ tok.hasMoreElements() ) { // throw away every other token
							String temp = tok.nextToken().trim();
                            if (temp.startsWith("'"))
                                temp = temp.substring(1);
                            if (temp.endsWith("'"))
                                temp = temp.substring(0, temp.length()-1);
                            
                            int index;
                            String temp2 = "";
                            while(((index=temp.indexOf("\\"))>=0) && (index<temp.length()-1)) {
                                temp2 += temp.substring(0, index);
                                temp = temp.substring(index+1);
                            }
                            temp2 += temp;
                            
							currentComps[compIndex].add(temp2);
						}
						
						line = strtok.nextToken();
						compIndex++;
						if (compIndex >= COMPS.length)
							break;
					}
				}
				
				/* create a hash set of items for quick searching */
				Set collectableOptions = new HashSet();
				collectableOptions.add(BUG_STATUS);
				collectableOptions.add(RESOLUTION);
				collectableOptions.add(SEVERITY);
				collectableOptions.add(PRIORITY);
				collectableOptions.add(PLATFORM);
				collectableOptions.add(OPERATING_SYSTEM);
				
				while (strtok.hasMoreTokens()) {
					/* find the lists of options */
					while ( (strtok.hasMoreTokens()) && (line.indexOf("name=\"")) < 0 )
						line = strtok.nextToken();
					
					if (!strtok.hasMoreTokens()) {
						break;
					}
					
					String id = line.substring(line.indexOf("name=\"") + 6); // 6 chars in name="
					id = id.substring(0, id.indexOf("\""));
					
					if (id.equals("product")) {
						/* parse product names from javascript */
						Iterator iter = allOptions.iterator();
						parseProducts(strtok, iter);
					}
					else {
						if (collectableOptions.contains(id)) {
							parseOption(id, strtok);
						}
					}
					line = strtok.nextToken();
				}
				monitor.worked(2);
				parseKeywords(monitor);
			}
			catch(BugzillaException e) {
				//System.err.println("Unable to contact bugzilla database");
				throw(e);
			} finally {
			monitor.done();
            }
		}

		/*
		 * Parse the values for the option named <code>id</code> from the string tokenized
		 * input stream <code>strtok</code>.
		 * 
		 * @param id The name identifying the option of which values will be parsed.
		 * @param strtok The string tokenized input.
		 */
		private void parseOption(String id, StringTokenizer strtok) {

			/* parse values and store in list */
			List values = new LinkedList();
			String optionFlag = "<option value=\"";
			String line;
			while ( (strtok.hasMoreTokens() && (line = strtok.nextToken()) != null) && line.indexOf("</select>") < 0 ) {
				while ( line.indexOf(optionFlag) < 0 ) {
					line = strtok.nextToken();
				}
				line = line.substring(line.indexOf(optionFlag) + optionFlag.length(), line.length());
				
				if (line.indexOf("\"") < 0) {
					break;
				}
				
				String value = line.substring(0, line.indexOf("\""));
				values.add(value);
			}
				
			alternativeOptions.put(id, values);
		}

		/*
		 * Parse the product named <code>id</code> list from the string tokenized input 
		 * stream <code>strtok</code>.  The product list is parsed seperatly from other 
		 * options because the products need to be mapped to their associated components.
		 * 
		 * @param strtok The string tokenized input.
		 * @param iter An iterator for the components to be associated with the product list.
		 */
		private void parseProducts(StringTokenizer strtok, Iterator iter) {
			String line;
			while ( (strtok.hasMoreTokens() && (line = strtok.nextToken()) != null) && line.indexOf("</select>") < 0 ) {
				while ( line.indexOf("<option value=") < 0 )
					line = strtok.nextToken();
				
				if( !iter.hasNext() ) {
					System.err.println("BugzillaQueryOptionParser: Incorrect number of products found");
					return;
				}
				
				List[] currentOption = (List[])iter.next();
				
				if(line.indexOf("\"") < 0 || line.indexOf("\"") == line.lastIndexOf("\""))
					break;
				
				String product = line.substring(line.indexOf("\"") + 1, line.lastIndexOf("\""));
				productComponentOptions.put(product, currentOption);
				
			}
		}
		
		private void parseKeywords(IProgressMonitor monitor) throws BugzillaException {
			String url = baseUrl;
			if ( !url.endsWith("/") )
				url += "/";
			
			url += URL_SUFFIX_KEYWORDS;
			
			String queryPage = (new HTTPConnection(url)).getString(monitor);
			if (queryPage == null || queryPage.equals("")) {
				System.err.println("BugzillaQueryOptionParser: Could not retrieve keywords");
				return;
			}
			
			StringTokenizer strtok = new StringTokenizer(queryPage, "\n");
			String id = "a name=\"";
			String line = "";
			
			while ( (line = strtok.nextToken()) != null) {
				int idIndex = -1;
				while (strtok.hasMoreTokens() && (idIndex = line.indexOf(id)) < 0) {
					line = strtok.nextToken();
				}
				
				if (!strtok.hasMoreTokens()) {
					return;
				}
				
				String keyword = line.substring(idIndex + id.length());
				keyword = keyword.substring(0, keyword.indexOf("\""));
				keywords.add(keyword);
			}
			monitor.worked(3);
		}
	}
	
	public String toString() {
		return "query options from " + getUrl();
	}
}
