/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.bugs.model.IGenericModel;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.datatypes.AttachmentDescription;
import org.eclipse.team.bugzilla.datatypes.CommentData;
import org.eclipse.team.bugzilla.datatypes.ReportData;

/**
 * 
 */
public class BugzillaReport extends BugzillaModel implements IRemoteModel {
    
    private final String fID;
    private List fComments = new ArrayList();

    public BugzillaReport(BugzillaContainer parent, String id) {
        super();
        fID= id;
        if (parent != null) {
            parent.addChildren(new BugzillaModel [] { this });
            buildComments();
        }
    }

    public boolean hasLocalContent() {
        return fComments.size() > 0;
    }

    public ReportData getReportData() {
        if (getBugzillaProvider() == null)
            return null;
        return getBugzillaProvider().getDataCache().getReport(fID);
    }
    
    public boolean hadErrors() {
        if (getBugzillaProvider() == null)
            return false;
        return getBugzillaProvider().getDataCache().hadErrorsForReport(fID);
    }
    
    public String getID() {
        return fID;
    }
    
    public List getComments() {
        return fComments;
    }

    public void setRefreshing(boolean refreshing) {
        super.setRefreshing(refreshing);
        for (Iterator iter = fComments.iterator(); iter.hasNext();) {
            ((IGenericModel)iter.next()).setRefreshing(refreshing);
        }
    }

    public void refresh(IProgressMonitor monitor) throws BugzillaException {
        BugzillaProvider bugzillaProvider = getBugzillaProvider();
        if (bugzillaProvider != null)
            bugzillaProvider.getDataCache().refreshReports(new String[] { getID() }, monitor);
    }
    
    public void update() {
        try {
	        buildComments();
        } finally {
            super.update();
        }
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#setParent(org.eclipse.team.bugzilla.model.BugzillaModel)
     */
    protected void setParent(BugzillaModel parent) {
        super.setParent(parent);
        buildComments();
    }
    
    private void buildComments() {
        final ReportData data= getReportData();
        if (data == null) return;
        fComments = new ArrayList();
        for (Iterator iter = data.comments.iterator(); iter.hasNext();) {
            final CommentData commentData= (CommentData) iter.next();
            // TODO: This should be done somewhere else
            if (commentData.attachmentDesc != null)
                commentData.attachmentDesc.set(AttachmentDescription.BUG_ID, this.getID());
            final BugzillaComment comment= new BugzillaComment(this, commentData);
            fComments.add(comment);
        }
        fireChildrenAdded((BugzillaModel [])fComments.toArray(new BugzillaComment [fComments.size()]));
    }

    public void accept(IBugzillaModelVisitor visitor) throws BugzillaException {
        if (visitor.visit(this)) {
            for (Iterator iter = fComments.iterator(); iter.hasNext();) {
                BugzillaComment element = (BugzillaComment) iter.next();
                element.accept(visitor);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.IRemoteModel#needsRefresh()
     */
    public boolean needsRefresh() {
        return getReportData() == null || getReportData().comments.size() == 0;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "report #" + fID;
    }
}


