/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.operations;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.datatypes.AttachmentData;
import org.eclipse.team.bugzilla.datatypes.AttachmentDescription;
import org.eclipse.team.bugzilla.datatypes.BugzillaConfiguration;
import org.eclipse.team.bugzilla.datatypes.QueryData;
import org.eclipse.team.bugzilla.datatypes.QueryURL;
import org.eclipse.team.bugzilla.datatypes.ReportData;
import org.eclipse.team.bugzilla.xml.Parser;
import org.eclipse.team.bugzilla.xml.QueryHandler;
import org.eclipse.team.bugzilla.xml.ReportHandler;
import org.eclipse.team.bugzilla.xml.ReportHandler_2_16;

/**
 * 
 */
public class BugzillaOperations_2_16 extends BugzillaOperations {
    
    private final static class Reader {
        
        private final String fSource;
        private int fIndex;
        
        public Reader(String source) {
            fSource= source;
            fIndex= 0;
        }
        
        public char read() {
            return fSource.charAt(fIndex++);
        }
        
        public boolean isFinished() {
            return fIndex == fSource.length();
        }
        
        public void consume(String string) throws BugzillaException {
            if (!fSource.startsWith(string, fIndex)) {
                throw new BugzillaException(BugzillaException.XML_ERROR, "Parse error consuming " + string);
            }
            fIndex += string.length();
        }
    }
    
    public ReportData getReportData(IProgressMonitor monitor, BugzillaConfiguration configuration, String id) throws BugzillaException {
        
        monitor.subTask("Fetching report data");
        try {
			String url = configuration.getBaseURL();
			if (!url.endsWith("/")) {
				url += "/";
			}
            url += bugScript + id.trim(); //$NON-NLS-1$
            final HTTPConnection connection= new HTTPConnection(url);
            final String xml= connection.getString(monitor);
            final ReportHandler handler= new ReportHandler_2_16();
            new Parser(false, false).parse(xml, handler);
            return handler.getReportData();
        } finally {
            monitor.done();
        }
    }
    
    public AttachmentData getAttachment(IProgressMonitor monitor, BugzillaConfiguration configuration, AttachmentDescription desc) throws BugzillaException {
        
        monitor.subTask("Fetching attachment data");
        try {
            final String id= desc.getString(AttachmentDescription.ID).trim();
            final String url= getAttachmentURL(configuration, id);
            final HTTPConnection connection= new HTTPConnection(url);
            final AttachmentData attachment= new AttachmentData();
            attachment.data= connection.getBytes(monitor);
            parse(connection.getContentType(), attachment);
            return attachment;
        } finally {
            monitor.done();
        }
    }
    
    private void parse(String contentType, AttachmentData data) throws BugzillaException {
        
        /**
         * The format we parse is like:
         *   text/plain; name="bug36204-1.txt"; charset=ISO-8859-1
         * or
         *   image/jpeg; name="text_editor_bug.jpg"
         *
         */
        final StringBuffer mimeType= new StringBuffer();
        final StringBuffer filename= new StringBuffer();
        final StringBuffer charset= new StringBuffer();
        
        final Reader reader= new Reader(contentType);
        
        /**
         * Read the MIME type.
         */
        char ch;
        while ((ch= reader.read()) != ';') {
            mimeType.append(ch);
        }
        
        reader.consume(" name=\"");
        
        /**
         * Read the file name
         */
        while ((ch= reader.read()) != '"') {
            filename.append(ch);
        }
        
        /**
         * Read the encoding if specified.
         */
        if (!reader.isFinished()) {
            reader.consume("; charset=");
            
            while (!reader.isFinished()) {
                charset.append(reader.read());
            }
        }
        data.set(AttachmentData.FILENAME, filename.toString());
        data.set(AttachmentData.MIMETYPE, mimeType.toString());
        data.set(AttachmentData.ENCODING, charset.toString());
    }
    
    public String getReportURL(BugzillaConfiguration configuration, String reportID) {
        return configuration.getBaseURL() + "/show_bug.cgi?id=" + reportID.trim();
    }
    
    public String getCommentURL(BugzillaConfiguration configuration, String reportID, int commentNbr) {
        return getReportURL(configuration, reportID) + "#c" + commentNbr;
    }
    
    public String getAttachmentURL(BugzillaConfiguration configuration, String attachmentID) {
        return configuration.getBaseURL() + "/attachment.cgi?id=" + attachmentID; //$NON-NLS-1$
    }
    
    public String getReportIDFromURL(BugzillaConfiguration configuration, String url) {
        if (!url.startsWith(configuration.getBaseURL() + "/show_bug.cgi?id="))
            return null;
        
        final int hashIndex = url.indexOf("#");
        final int equalsIndex = url.indexOf("=");
        try {
            if(hashIndex == -1)
                return url.substring(equalsIndex + 1);
            return url.substring(equalsIndex + 1, hashIndex);
        } catch (ArrayIndexOutOfBoundsException e) {
            return null;
        }
    }
    
    
    public String getQueryURL(BugzillaConfiguration configuration, String url) {
        if (!url.startsWith(configuration.getBaseURL() + "/buglist.cgi?"))
            return null;
        
        if (url.indexOf("regetlastlist") != -1)
            return null;
        return url;
    }

    /**
     * Fetch the query data for the given query from the server identified by the supplied
     * configuration. This method uses the query handler returned from <code>getQueryHandler</code>
     * to perform the parsing.
     * @param monitor a progress monitor
     * @param configuration the configuration which identified the bugzilla server
     * @param queryURL the query to be fetched
     * @return the data retrieved for the given query
     * @throws BugzillaException
     */
    public QueryData getQueryData(IProgressMonitor monitor, BugzillaConfiguration configuration, QueryURL queryURL) throws BugzillaException {
        final QueryHandler handler= new QueryHandler();
    
        String url= queryURL.getUrl(configuration); //$NON-NLS-1$
        // catch query data triggered by the browser (non-xml)
        // this also prevents queries from being added to the history
        if (url.indexOf("format=rdf") < 0) {
        	return null;
        }
        final String loginCookie = configuration.getLoginCookie();
        if (queryURL.isNamedQuery() && loginCookie == null) {
            throw new BugzillaException(BugzillaException.REQUIRES_LOGIN, "Please login to Bugzilla.");
        }
        final HTTPConnection connection= new HTTPConnection(url, loginCookie);
        final String xml= connection.getString(monitor);
        if (isLoginFailure(xml)) {
            throw new BugzillaException(BugzillaException.REQUIRES_LOGIN, "Bugzilla user authentication failed.");
        }
        new Parser(false, false).parse(xml, handler);
        return handler.getQueryData();
    }

    public boolean isLoginFailure(String xml) {
        return xml.indexOf("<html>") != -1 && xml.indexOf("Login") != -1; //$NON-NLS-1$ //$NON-NLS-2$
    }
}
