/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.operations;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;

import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.Util;
import org.eclipse.team.bugzilla.model.BugzillaProvider;

/**
 * Log into a bugzilla server
 */
public class LoginOperation {

    private static final String DATA_ENCODING = "UTF-8"; //$NON-NLS-1$
    
    public void run(BugzillaProvider provider, String email, String password) throws BugzillaException {
        OutputStreamWriter out = null;
        try {
            String data = getLoginData(email, password);
            URL url = getLoginUrl(provider);
            URLConnection connection = open(url);
            out = sendData(connection, data);
            String loginCookie = getLoginCookie(connection);
            dumpPage(connection);
            if (loginCookie == null) 
                throw new BugzillaException(BugzillaException.GENERAL_ERROR, "Login failed for unknown reason");
            provider.getConfiguration().setLoginCookie(loginCookie);
            
        } finally {
            close(out);
        }
    }
    
    private URLConnection open(URL url) throws BugzillaException {
        try {
            return url.openConnection();
        } catch (IOException e) {
            throw new BugzillaException(BugzillaException.IO_ERROR, "IO error opening " + e.getMessage(), e);
        }
    }

    private void dumpPage(URLConnection connection) {
        BufferedReader in = null;
        try {
            in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            while (in.readLine() != null) {
                // We don't want the page contents
            }
        } catch (IOException e) {
            // Log it and continue since we don't want the 
            BugzillaPlugin.log(new BugzillaException("IO error while reading " + e.getMessage(), e)); //$NON-NLS-1$
        } finally {
            if (in != null)
                try {
                    in.close();
                } catch (IOException e1) {
                    // Ignore
                }
        }
    }

    private OutputStreamWriter sendData(URLConnection connection, String data) throws BugzillaException {
        OutputStreamWriter out = null;
        try {
            connection.setDoOutput(true);
            out = new OutputStreamWriter(connection.getOutputStream());
            out.write(data);
            out.flush();
            return out;
        } catch (IOException e) {
            close(out);
            throw new BugzillaException("An error occurred sending the login data: " + e.getMessage(), e);
        }
    }

    private void close(OutputStreamWriter out) {
        try {
            if (out != null) out.close();
        } catch (IOException e1) {
            // Ignore
        }
    }

    private URL getLoginUrl(BugzillaProvider provider) throws BugzillaException {
        String urlString = Util.appendPath(provider.getConfiguration().getBaseURL(), "query.cgi"); //$NON-NLS-1$
        try {
            return new URL(urlString); //$NON-NLS-1$
        } catch (MalformedURLException e) {
            throw new BugzillaException("Invalid URL: " + urlString, e);
        }
    }

    private String getLoginData(String email, String password) throws BugzillaException {
        try {
            StringBuffer data = new StringBuffer();
            data.append(URLEncoder.encode("Bugzilla_login", DATA_ENCODING)); //$NON-NLS-1$
            data.append("="); //$NON-NLS-1$
            data.append(URLEncoder.encode(email, DATA_ENCODING));
            data.append("&");
            data.append(URLEncoder.encode("Bugzilla_password", DATA_ENCODING)); //$NON-NLS-1$
            data.append("="); //$NON-NLS-1$
            data.append(URLEncoder.encode(password, DATA_ENCODING));
            data.append("&");
            data.append(URLEncoder.encode("GoAheadAndLogIn", DATA_ENCODING)); //$NON-NLS-1$
            data.append("="); //$NON-NLS-1$
            data.append(URLEncoder.encode("1", DATA_ENCODING)); //$NON-NLS-1$
            return data.toString();
        } catch (UnsupportedEncodingException e) {
            throw new BugzillaException("Unsupported encoding: " + DATA_ENCODING, e);
        }
    }

    private String getLoginCookie(URLConnection connection) {
        String login = null;
        String password = null;
        for (int i=0; ; i++) {
            String name = connection.getHeaderFieldKey(i);
            String value = connection.getHeaderField(i);
            if (name == null && value == null) {
                // No more headers
                break;
            }
            if ("Set-Cookie".equalsIgnoreCase(name)) { //$NON-NLS-1$
                int index = value.indexOf(";");
                String cookie = value.substring(0, index).trim();
                if (cookie.startsWith("Bugzilla_logincookie")) {
                    password = cookie;
                } else if (cookie.startsWith("Bugzilla_login")) {
                    login = cookie;
                }
            }
        }
        if (login != null && password != null) {
            return login + "; " + password;
        }
        return null;
    }
}
