/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.operations;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.actions.LoginAction;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.eclipse.team.bugzilla.model.BugzillaProvider;
import org.eclipse.team.bugzilla.model.IRemoteModel;
import org.eclipse.team.ui.TeamOperation;
import org.eclipse.ui.IWorkbenchPart;

/**
 * Refreshes bugzilla model elements
 */
public class RefreshOperation extends TeamOperation {
    
    private static class UIUpdateTool implements Runnable {
        private IRemoteModel fElement;
        private boolean fRefresh;
        
        public void run() {
            fElement.setRefreshing(fRefresh);
        }
        
        public void setRefreshing(IRemoteModel element, boolean refresh) {
            fElement= element;
            fRefresh= refresh;
            Display.getDefault().syncExec(this);
        }
    }
    
    private static class ErrorCollector {
        private final List fErrors;
        
        ErrorCollector() {
            fErrors= new ArrayList();
        }
        
        void add(IStatus status) {
            fErrors.add(status);
        }
        
        IStatus getReturnStatus(IProgressMonitor monitor) {
            if (fErrors.size() == 1) {
                return (IStatus)fErrors.get(0);
            } else if (!fErrors.isEmpty()) {
                return new MultiStatus(BugzillaPlugin.ID, BugzillaException.MULTI_ERROR, (IStatus[]) fErrors.toArray(new IStatus[fErrors.size()]), "Errors occurred while refreshing", null);
            }
            return monitor.isCanceled() ? Status.CANCEL_STATUS : Status.OK_STATUS;
        }
        
        /**
         * @param e
         */
        public void handle(BugzillaException e, IRemoteModel element) {
            final IStatus status= e.getStatus();
            switch (status.getCode()) {
            case BugzillaException.XML_ERROR:
            {
                final String msg= "Error fetching " + element.toString() + ": Could not interpret the received data. Maybe the Bugzilla URL is invalid or the Bugzilla version is not supported.";
                add(new Status(IStatus.ERROR, BugzillaPlugin.ID, status.getCode(), msg, e));
                break;
            }
            case BugzillaException.REQUIRES_LOGIN:
            case BugzillaException.IO_ERROR:
            case BugzillaException.UNEXPECTED_CONDITION:
            case BugzillaException.PERSISTENCE_READ_ERROR:
            case BugzillaException.MULTI_ERROR:
            {
            	String message = element.toString();
        		int index = message.indexOf("?");
        		message = (index > 0) ? message.substring(0, index+1) : message;
            	
                final String msg= "Error fetching " + message + ": " + status.getMessage();
                add(new Status(IStatus.ERROR, BugzillaPlugin.ID, status.getCode(), msg, e));
                break;
            }
            }
        }
    }
    
    private final UIUpdateTool fUpdateTool;
    private final ErrorCollector fErrorCollector;
    
    private final IRemoteModel[] fElements;
    
    public RefreshOperation(IWorkbenchPart part, IRemoteModel[] elements) {
        super(part);
        fUpdateTool= new UIUpdateTool();
        fErrorCollector= new ErrorCollector(); 
        fElements = elements;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
        monitor.beginTask("Retrieving data from server", 10 * fElements.length);
        try {
            for (int i = 0; i < fElements.length; i++) {
                fetchElement(fElements[i], monitor);
            } 
        }finally {
            monitor.done();
        }
        final IStatus returnStatus = fErrorCollector.getReturnStatus(monitor);
        if (!returnStatus.isOK())
            throw new InvocationTargetException(new BugzillaException(returnStatus));
    }
    
    /**
     * 
     */
    private void fetchElement(IRemoteModel element, IProgressMonitor monitor) {
        try {
            fetch(element, new SubProgressMonitor(monitor, 10));
        } catch (BugzillaException e) {
            final IStatus status= e.getStatus();
            if (status.getCode() == BugzillaException.REQUIRES_LOGIN) {
                try {
                    if (!promptForAuthentication(element, e.getMessage())) {
                        throw new OperationCanceledException();
                    }
                    fetch(element, monitor);
                    return;
                } catch (BugzillaException e1) {
                    fErrorCollector.add(e.getStatus());
                }
            } else {
                fErrorCollector.handle(e, element);
            }
        }
    }
    
    private void fetch(IRemoteModel element, IProgressMonitor monitor) throws BugzillaException {
        fUpdateTool.setRefreshing(element, true);
        try {
            element.refresh(new SubProgressMonitor(monitor, 10));
        } finally {
            fUpdateTool.setRefreshing(element, false);
        }
    }
    
    private boolean promptForAuthentication(IRemoteModel element, final String message) throws BugzillaException {
        final BugzillaProvider provider = ((BugzillaModel)element).getBugzillaProvider();
        if (provider == null) {
            // The element has been removed from the model
            return false;
        }
        final boolean[] success = new boolean[] { false };
        final BugzillaException[] exception = new BugzillaException[] { null };
        Display.getDefault().syncExec(new Runnable() {
            public void run() {
                try {
                    success[0] = LoginAction.performLogin(getShell(), provider, message);
                } catch (BugzillaException e) {
                    exception[0] = e;
                }
            }
        });
        if (exception[0] != null) {
            throw exception[0];
        }
        return success[0];
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.ui.TeamOperation#canRunAsJob()
     */
    protected boolean canRunAsJob() {
        return true;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.ui.TeamOperation#isUserInitiated()
     */
    public boolean isUserInitiated() {
        return false;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.ui.TeamOperation#getJobName()
     */
    protected String getJobName() {
        return "Retrieving Bugzilla data";
    }
}
