/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.persistence;

import org.eclipse.team.bugs.extension.BugsException;
import org.eclipse.team.bugs.tools.Utils;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.model.BugzillaContainer;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;

/**
 * An abstract class which defines the protocol for persisting
 * a bugzilla model and provides several helper methods.
 */
public abstract class AbstractPersistenceAdapter implements IPersistenceAdapter {
    
    private static final String URL = "org.eclipse.team.bugzilla.url"; //$NON-NLS-1$
    
    protected static final BugzillaModel [] EMPTY_ARRAY= new BugzillaModel[0];
    protected static final String EMPTY_STRING= Utils.EMPTY_STRING;
    
    /**
     * Return the preference node that represents the given 
     * element in the preference store. The location of
     * the node is determined by the element's path.
     * @param element the element to be persisted
     * @return the node in which the element shoudl be persisted
     */
    public static final Preferences getPreferences(Object element) {
        /**
         * At this time, the element may not have a parent anymore as it has already been removed
         * from it. So we need to have another way to find the node!
         */
        if (element == null || !(element instanceof BugzillaModel)) {
            return getRootPreferences();
        }
        BugzillaModel bugzillaModel = (BugzillaModel)element;
        String path = bugzillaModel.getPath();
        return getRootPreferences().node(path);
    }
    
    /**
     * This method should return an ID which is unique among all other elements
     * with the same parent in the tree. Subclasses must implement this.
     * The id for a particular element should nopt change during it's
     * lifetime.
     * @param element the element being persisted
     * @return An id unique among all other elements with the same parent
     * which doesn't change during the life of the element.
     */
    public final String getId(Object element) {
        if (element instanceof BugzillaModel)
            return ((BugzillaModel)element).getID();
        return null;
    }

    private static Preferences getRootPreferences() {
        return PersistenceManager.getRootNode();
    }
    
    /**
     * Return the children of the given element.
     * @param element the parent element
     * @return the children of the element
     */
    protected BugzillaModel[] getChildren(BugzillaModel element) {
        if (element instanceof BugzillaContainer) {
            return ((BugzillaContainer)element).getChildren();
        }
        return new BugzillaModel[0];
    }
    
    public void purgeChild(BugzillaModel element, BugzillaModel child, boolean flush) {
        final Preferences node= getPreferences(element);
        if (node == null) 
            return;
        
        final String id= child.getID();
        
        final Preferences childNode= node.node(id);
        try {
            childNode.removeNode();
        } catch (BackingStoreException e) {
            BugzillaPlugin.log(BugsException.asBugsException(e));
        } finally {
            if (flush)
                flush(node);
        }
    }
    
    /**
     * Store the parameters of the model object in the preferences node.
     * By default, the element type and id are stored. Subclasses may
     * override to store additional information.
     * @param element the element to be stored
     * @param node The preferences node where the object is stored.
     */
    protected void store(BugzillaModel element, Preferences node) {
        node.put(PersistenceManager.MODEL_TYPE, getType());
        node.put(PersistenceManager.ELEMENT_ID, getId(element));
    }
    
    /**
     * Return the type that is handled by this persistence adapter.
     * @return the type that is handled by this persistence adapter
     */
    protected abstract String getType();

    /**
     * Rstore the element that was previously stored in the given node.
     * @param parent the parent element
     * @param node the node to which the element was stored
     * @param id
     * @return the restored element
     * @throws BugzillaException
     */
    protected abstract BugzillaModel restore(BugzillaContainer parent, Preferences node, String id) throws BugzillaException;
    
    public void storeElement(BugzillaModel element, boolean flush) {
        final Preferences node= getPreferences(element);
        if (node == null) 
            return;

        // Store the element
        store(element, node);
        
        // Store any elegable children
        final BugzillaModel[] children = getChildren(element);
        for (int i = 0; i < children.length; i++) {
            final BugzillaModel child = children[i];
            final IPersistenceAdapter childAdapter = PersistenceManager.getPersistenceAdapter(child);
            if (childAdapter != null) {
                childAdapter.storeElement(child, false /* don't flush */);
            }
        }
        // Write to disk is requested
        if (flush)
            flush(node);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.persistence.IPersistenceAdapter#restoreElement(org.eclipse.team.bugzilla.model.BugzillaModel, org.osgi.service.prefs.Preferences)
     */
    public BugzillaModel restoreElement(BugzillaContainer parent, Preferences node, String id) throws BugzillaException {
        BugzillaModel element = restore(parent, node, id);
        if (element instanceof BugzillaContainer)
            PersistenceManager.restoreChildren((BugzillaContainer)element, node);
        return element;
    }

    private void flush(final Preferences node) {
        try { 
            node.flush(); 
        } 
        catch (BackingStoreException e) {
            BugzillaPlugin.log(BugsException.asBugsException(e));
        }
    }
    
    protected static boolean empty(String string) {
        return string == null || EMPTY_STRING.equals(string.trim());
    }
    
    /**
     * Helper method for storing a URL.
     * @param node the node in which to store the URL
     * @param url the url to be stored
     */
    protected void storeUrl(Preferences node, String url) {
        node.put(URL, url);
    }
    

    /**
     * Helper method for retieving URLs that were store using
     * <code>storeUrl</code>
     * @param node the node for the element
     * @return the previously stored URL or <code>null</code>
     */
    protected String getUrl(Preferences node) {
        return node.get(URL, null);
    }
}
