/**
 * 
 * @author Igor Foox <ifoox@redhat.com>
 */

package org.eclipse.team.bugzilla.testing;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.model.BugzillaQueryOptions;

/**
 * Defines the query options for a <code>MockBugzillaServer</code>. This
 * implementation does nothing in the <code>refresh(IProgressMonitor)</code>
 * method, because the options are local and thus can never be out of date.
 * 
 * In addition this class provides method for adding Products, Components, and
 * other bugzilla options. This is unavailable in <code>BugzillaQueryOptions</code>
 * because it cannot control a Bugzilla installation. However the 
 * <code>MockBugzillaServer</code> needs a way of adding to the query options.
 */
public class MockBugzillaQueryOptions extends BugzillaQueryOptions {

	public MockBugzillaQueryOptions(String queryPage) {
		super(queryPage);
	}

	/**
	 * Does nothing except signaling to <code>monitor</code> that it has 
	 * done it's job.
	 */
	public void refresh(IProgressMonitor monitor) throws BugzillaException {
		// do nothing because this local mock bugzilla can never be out of date
		monitor.done();
	}
	
	/**
	 * Set the list of options associated with the given value to the given 
	 * <code>List</code>. This method should NOT be used to add products and
	 * product options, instead the <code>addProduct(String)</code> and
	 * <code>addProductXXX(String, String)</code> methods should be used for that
	 * purpose.
	 *  
	 * @param optName The vlaue with which options will be associated
	 * @param optValue The list of options to associate with the govem value
	 */
	public void setAlternativeOptions(String optName, List optValues) {
		setOptions(optName, optValues);
	}
	
	/**
	 * Add an option to the options associated with the given value if the
	 * option is not already in the set. This method should NOT be used to add 
	 * products and product options, instead the <code>addProduct(String)</code>
	 * and <code>addProductXXX(String, String)</code> methods should be used 
	 * for that purpose.
	 * 
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the alternative options list.
	 * 
	 * @param optName The vlaue with which the option will be associated
	 * @param optValue The option to associate with the given value
	 */
	public void addAlternativeOption(String optName, String optValue) {
		addOption(optName,optValue);
	}
	
	/**
	 * Remove an option from the options associated with the given value. If no
	 * such option exists this method has no effect. If the list of options
	 * associated with the value becomes empty this removes the value altogether.
	 * This method should NOT be used to remove product options, instead the
	 * <code>removeXXX(String, String)</code> methods should be used for 
	 * that purpose.
	 * 
	 * This method is provided so that subclasses that do not parse query
	 * options in the same way can manipulate the alternative options list.
	 * 
	 * @param optName The value with from whos associated options the given 
	 * option will be removed
	 * @param optValue The option to remove
	 */
	public void removeAlternativeOption(String optName, String optValue) {
		removeOption(optName, optValue);
	}
	
	/**
	 * Removes the given value and all of its associated options from the set
	 * of alternative options. If the value is not in the set, this method has 
	 * no effect. This method should NOT be used to remove products, instead
	 * the <code>removeProduct(String)</code> method should be used for that 
	 * purpose.
	 *  
	 * @param optName The value to be removed from the set of alternative options
	 */
	public void unsetAlternativeOptions(String optName) {
		unsetOptions(optName);
	}

	/**
	 * Add a new product definition to the query options. This method has
	 * no effect if the product already exists.
	 * 
	 * @param name The name of the product to add
	 */
	public void addProduct(String name) {
		super.addProduct(name);
	}

	/**
	 * Remove a product definition from the query options. This method has no
	 * effect if the product is not defined.
	 *
	 * @param name The name of the product to remove
	 */
	public void removeProduct(String name) {
		super.removeProduct(name);
	}
	
	/**
	 * Add a component to an existing product. This method has no efect
	 * if the given product is undefined or the given component already
	 * exists for the given product.
	 * 
	 * @param product The name of the product to which to add a component
	 * @param component The name of the component to add
	 */
	public void addProductComponent(String product, String component){
		List opts[] = getProductOptions(product);
		if (opts == null)
			return;
		
		// No repetition of components allowed
		if (opts[COMPONENT_INDEX].contains(component))
			return;
		
		opts[COMPONENT_INDEX].add(component);
	}
	
	/**
	 * Checks if the given product contains a component with the given name.
	 * 
	 * @param product The product to check for the existance of a component
	 * @param component The component whose existence is checked
	 * @return <code>true</code> if the given component exists in the given 
	 * product, <code>false</code> otherwise
	 */
	public boolean hasComponent(String product, String component) {
		return getProductOptions(product)[COMPONENT_INDEX].contains(component);
	}
	
	/**
	 * Remove a component from a product definition. This method has no effect
	 * if the product does not exist or the component does not exist for the
	 * given product.
	 * The product itself is not deleted even if there are no other components
	 * (or other options) defined for it. It must be removed by calling
	 * <code>removeProduct(String)</code>.
	 * 
	 * @param product The product from which to remove a component
	 * @param component The component definition to remove
	 */
	public void removeComponent(String product, String component) {
		List[] prodOpts = getProductOptions(product);
		if (prodOpts == null)
			return;
		
		prodOpts[COMPONENT_INDEX].remove(component);
	}
	
	
	
	/**
	 * Add a version to an existing product. This method has no effect
	 * if the given product is undefined or the given version already
	 * exists for the given product.
	 * 
	 * @param product The name of the product to which to add a version
	 * @param version The name of the version to add
	 */
	public void addProductVersion(String product, String version){
		List opts[] = getProductOptions(product);
		if (opts == null)
			return;
		
		// No repetition of versions allowed
		if (opts[VERSION_INDEX].contains(version))
			return;
		
		opts[VERSION_INDEX].add(version);
	}
	
	/**
	 * Checks if the given product contains a version with the given name.
	 * 
	 * @param product The product to check for the existance of a version
	 * @param version The component whose existence is checked
	 * @return <code>true</code> if the given version exists in the given 
	 * product, <code>false</code> otherwise
	 */
	public boolean hasVersion(String product, String version) {
		return getProductOptions(product)[VERSION_INDEX].contains(version);
	}
	
	/**
	 * Remove a version from a product definition. This method has no effect
	 * if the product does not exist or the version does not exist for the
	 * given product.
	 * The product itself is not deleted even if there are no other versions
	 * (or other options) defined for it. It must be removed by calling
	 * <code>removeProduct(String)</code>.
	 * 
	 * @param product The product from which to remove a version
	 * @param version The version definition to remove
	 */
	public void removeVersion(String product, String version) {
		List[] prodOpts = getProductOptions(product);
		if (prodOpts == null)
			return;
		
		prodOpts[VERSION_INDEX].remove(version);
	}

	
	/**
	 * Add a target milestone to an existing product. This method has no efect
	 * if the given product is undefined or the given target milestone already
	 * exists for the given product.
	 * 
	 * @param product The name of the product to which to add a target milestone
	 * @param targetMilestone The name of the target milestone to add
	 */
	public void addProductTargetMilestone(String product, String targetMilestone){
		List opts[] = getProductOptions(product);
		if (opts == null)
			return;
		
		// No repetition of target milestones allowed
		if (opts[TARGETMILESTONE_INDEX].contains(targetMilestone))
			return;
		
		opts[TARGETMILESTONE_INDEX].add(targetMilestone);
	}
	
	/**
	 * Checks if the given product contains a target milestone with the given 
	 * name.
	 * 
	 * @param product The product to check for the existance of a target 
	 * milestone
	 * @param targetMilestone The target milestone whose existence is 
	 * checked
	 * @return <code>true</code> if the given target milestone exists in the
	 * given product, <code>false</code> otherwise
	 */
	public boolean hasTargetMilestone(String product, String targetMilestone) {
		return getProductOptions(product)[TARGETMILESTONE_INDEX].contains(targetMilestone);
	}

	/**
	 * Remove a target milestone from a product definition. This method has 
	 * no effect if the product does not exist or the target milestone does
	 * not exist for the given product.
	 * The product itself is not deleted even if there are no other target
	 * milestones (or other options) defined for it. It must be removed by
	 * calling <code>removeProduct(String)</code>.
	 * 
	 * @param product The product from which to remove a target milestone
	 * @param targetMilestone The target milestone definition to remove
	 */
	public void removeTargetMilestone(String product, String targetMilestone) {
		List[] prodOpts = getProductOptions(product);
		if (prodOpts == null)
			return;
		
		prodOpts[TARGETMILESTONE_INDEX].remove(targetMilestone);
	}


	

}
