/**
 * TODO: Copyright
 */
package org.eclipse.team.bugzilla.testing;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.datatypes.ReportData;

/**
 * A standalone, in-memory, mock Bugzilla server. This can be used to feed data
 * to the bugzilla plugin without running or connecting to a real Bugzilla 
 * server. This must be used in conjunction with 
 * <code>BugzillaOperations_Mock</code> to retrieve and store information.
 * 
 * @author Igor Foox <ifoox@redhat.com>
 */
public class MockBugzillaServer {
	/** Maximum value of a bug id */
	private static final int MAX_ID = Integer.MAX_VALUE;
	
	/** The hightest id currently given out */
	private int fNextId;

	/** All bugs in the bugzilla */
	protected Vector fBugs;
	
	/** All registered users in the system, mapped as 
	 * (<code>String</code> username) -> <code>MockBugzillaUser</code>
	 */
	protected Map fUsers;

	public MockBugzillaServer(){
		fBugs = new Vector();
		fUsers = Collections.synchronizedMap(new HashMap());
		fNextId = 0;
	}
	
	/**
	 * Add a new User witht he given information to the list of 
	 * users that are able to login to this Bugzilla server. There
	 * is currently no way to give users varying permissions and so a 
	 * user can do anything if it is added to the Bugzilla server.
	 * 
	 *  This method is synchronized because users must be unique.
	 *  
	 * @param username the desired username
	 * @param pass the optional password, ignored if <code>null</code>
	 * @param email the optional email, does not have to be a valid email, ignored if 
	 * <code>null</code>
	 * 
	 * @return a new instance of <code>MockBugzillaUser</code> corresponding to
	 * the newly added user, or <code>null</code> if such a user already exists.
	 */
	public synchronized MockBugzillaUser addUser(String username, String pass,
			String email) {
		if (fUsers.keySet().contains(username))
			return null;

		MockBugzillaUser newUser = new MockBugzillaUser(username, pass, email);
		fUsers.put(username, newUser);
		return newUser;
	}
	
	/**
	 * Returns the <code>MockBugzillaUser</code> associated with the given
	 * name.
	 * 
	 * @param username the username
	 * @return the <code>MockBugzillaUser</code> associated with the given
	 * name, or <code>null</code> if no such user exists
	 */
	public MockBugzillaUser getUser(String username){
		if (!fUsers.keySet().contains((username)))
			return null;
		
		return (MockBugzillaUser) fUsers.get(username);
	}
	
	/**
	 * Returns all of the bugs in the form of <code>ReportData</code> objects.
	 * 
	 * @return all of the bugs in the form of <code>ReportData</code> objects.
	 */
	public ReportData[] getAllBugs(){
		return (ReportData[])fBugs.toArray();
	}

	/**
	 * Returns the bug associated with the given id or <code>null</code> if none exist.
	 * 
	 * @param id a bug id
	 * @return he bug associated with the given id or <code>null</code> if none exist.
	 */
	public ReportData betBug(int id){
		if (id < fNextId)
			return null;
		
		return null;
		
	}

	/**
	 * Create a new bug report returning the id.
	 * 
	 * @return the id of the created bug report
	 * @throws BugzillaException if a new bug report could not be created
	 */
	public int newBug() throws BugzillaException{
		if (fNextId == MAX_ID)
			throw new BugzillaException(0, "Bugzilla bug limit exceeded.");
		
		int currentId = fNextId++;
		ReportData bug = new ReportData();
		fBugs.add(currentId, bug);
		
		return currentId;
	}
	
	/**
	 * Return the number of bug reports currently in the bugzilla.
	 * 
	 * @return the number of bug reports currently in the bugzilla.
	 */
	public int bugCount(){
		return fBugs.size();
	}

	/**
	 * Returns the bug report associated with the given bug id or null if one
	 * does not exist.
	 * 
	 * @param id the bug report id
	 * @return the bug report associated with the given id
	 */
	public ReportData getBugById(int id) {
		if (id > fNextId - 1)
			return null;
		
		return (ReportData) fBugs.get(id);
	}
	
	
}
