/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.view;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.jface.text.TextPresentation;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.team.bugzilla.datatypes.AttachmentData;
import org.eclipse.team.bugzilla.datatypes.AttachmentDescription;
import org.eclipse.team.bugzilla.datatypes.CommentData;
import org.eclipse.team.bugzilla.datatypes.ReportData;
import org.eclipse.team.bugzilla.model.BugzillaAttachment;
import org.eclipse.team.bugzilla.model.BugzillaComment;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.eclipse.team.bugzilla.model.BugzillaQuery;
import org.eclipse.team.bugzilla.model.BugzillaReport;

public class BugzillaModelPresenter {
    
    protected TextPresentation fPresentation;
    protected StringBuffer fBuffer;
    protected StyledText fStyledText;

    public BugzillaModelPresenter() {
        fStyledText= null;
        fPresentation= null;
        fBuffer= null;
    }
    
    private void begin(StyledText styledText) {
        fPresentation= new TextPresentation();
        fBuffer= new StringBuffer();
        fStyledText= styledText;
        
    }
    
    private void end(final boolean wordWrap) {
        BusyIndicator.showWhile(fStyledText.getDisplay(), new Runnable() {
            public void run() {
                fStyledText.setRedraw(false);
                fStyledText.setText("");
                fStyledText.setWordWrap(wordWrap);
                fStyledText.setText(fBuffer.toString());
                TextPresentation.applyTextPresentation(fPresentation, fStyledText);
                fStyledText.setRedraw(true);
            }
        });

        fBuffer= null;
        fPresentation= null;
        fStyledText= null;
    }
    
//    public void present(BugzillaProvider provider, StyledText styledText) {
//        begin(styledText);
//        end(true);
//    }

    public void present(BugzillaQuery query, StyledText styledText) {
        begin(styledText);
        append(query);
        end(true);        
    }
    
    public void present(BugzillaReport report, StyledText styledText) {
        begin(styledText);
        append(report);
        end(true);        
    }
    
    public void present(BugzillaComment comment, StyledText styledText) {
        begin(styledText);
        append(comment);
        end(true);
    }
    
    public void present(BugzillaAttachment bugzillaAttachment, StyledText styledText) {
        
        final AttachmentData attachment= bugzillaAttachment.getData();
        
        boolean wordWrap= false;
        begin(styledText);
        
        if (attachment != null) { 
            final String mimeType= bugzillaAttachment.getData().getString(AttachmentData.MIMETYPE);
            if (mimeType.startsWith("text/")) {
                final String data= bugzillaAttachment.getDataAsString();
                append(data);
                wordWrap= false;
            } else {
                append(attachment.getString(AttachmentData.FILENAME)).space().append("[").append(mimeType).append("]");
                wordWrap= true;
            }
        }
        end(wordWrap);
    }
    
//	/**
//	 * @param folder
//	 * @param styledText
//	 */
//	public void present(BugzillaFolder folder, StyledText styledText) {
//		begin(styledText);
//		append(folder);
//		end(true);
//	}
    
//    /**
//	 * @param folder
//	 */
//	private void append(BugzillaFolder folder) {
//		
//		bold(folder.getName()).endl().endl();
//		
//		BugzillaModel[] children = folder.getChildren();
//		int numQueries = 0;
//		int numBugs = 0;
//		if (children != null) {
//			for (int i = 0; i < children.length; i++) {
//				if (children[i] instanceof BugzillaQuery) {
//					numQueries++;
//				}
//				else {
//					numBugs++;
//				}
//			}
//		}
//		else {
//			italic("Empty Folder");
//			return;
//		}
//				
//		if (numQueries > 0) {
//			append("Queries:\t" + numQueries).endl();
//		}
//		if (numBugs > 0) {
//			append("Bugs:\t\t" + numBugs);
//		}
//	}

	private BugzillaModelPresenter append(BugzillaQuery query) {
        
    	String queryUrl = query.getQueryUrl().toUrlString();
        String summary = null;
        String comment = null;
        
        int summaryIndex = queryUrl.indexOf("short_desc=");
        if (summaryIndex >= 0) {
        	summary = queryUrl.substring(summaryIndex, queryUrl.length());
        	int ampIndex = summary.indexOf('&');
        	if (ampIndex < 0) {
        		ampIndex = summary.length();
        	}
        	try {
				summary = URLDecoder.decode(summary.substring(summary.indexOf('=') + 1, ampIndex), "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
        }
        
        int commentIndex = queryUrl.indexOf("long_desc=");
        if (commentIndex >= 0) {
        	comment = queryUrl.substring(commentIndex,  queryUrl.length());
        	int ampIndex = comment.indexOf('&');
        	if (ampIndex < 0) {
        		ampIndex = comment.length();
        	}
        	try {
				comment = URLDecoder.decode(comment.substring(comment.indexOf('=') + 1, ampIndex), "UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
        }
        
        BugzillaModel[] children = query.getChildren();
        HashMap bugFreqs = new HashMap(7); // 7 bugzilla status'
        int criticalBugs = 0;
        
        for (int i = 0; i < children.length; i++) {
        	ReportData reportData = ((BugzillaReport)children[i]).getReportData();
        	if (reportData == null) {
        		return this;
        	}
        	String status = reportData.getString(ReportData.STATUS);
        	
        	if (reportData.getString(ReportData.SEVERITY).equals("critical")) {
        		criticalBugs++;
        	}
        	
        	Integer count = (Integer)bugFreqs.get(status);
        	if (count == null) {
        		bugFreqs.put(status, new Integer(1));
        	}
        	else {
        		bugFreqs.put(status, new Integer(count.intValue() + 1));
        	}
        }
        
        bold("Query from " + query.getQueryUrl().getBaseUrl()).endl();
        if (summary != null && !summary.equals("")) {
        	append("Summary:\t\t").italic(summary).endl();
        }
        if (comment != null && !comment.equals("")) {
        	append("Comment:\t\t").italic(comment).endl();
        }
        endl();
        
        append("Total Bugs:\t" + children.length).endl();
        Iterator iter = bugFreqs.keySet().iterator();
        while (iter.hasNext()) {
        	String status = (String)iter.next(); 
        	String tab = "\t";
        	int statusLength;
        	if ( (statusLength = status.length()) < 8) {
        		tab += "\t";
        		if (statusLength < 4) {
        			tab += "\t";
        		}
        	}
        	append(status + tab + bugFreqs.get(status)).endl();
        }
        endl();
        if (criticalBugs > 0) {
        	append("This query contains").bold(" "+criticalBugs).append(" critical " + ((criticalBugs > 1) ? "bugs" : "bug"));
        }
        
        return this;
    }
    
    private BugzillaModelPresenter append(BugzillaComment bugzillaComment) {
        final CommentData comment= bugzillaComment.getCommentData();
        if (comment == null)
            return this;
        bold(comment.getString(CommentData.AUTHOR)).append(" - ").append(comment.getDate(CommentData.DATE)).endl(2);

        if (comment.attachmentDesc != null && comment.attachmentDesc.getBoolean(AttachmentDescription.OBSOLETE))
            append("*** This attachment is obsolete ***").endl(2);
        append(comment.getString(CommentData.TEXT));
        return this;
    }

    private BugzillaModelPresenter append(BugzillaReport bugzillaReport) {
        
        if (!bugzillaReport.hasLocalContent())
            return this;

        final ReportData report= bugzillaReport.getReportData();
        
        bold("[").bold(report.getString(ReportData.ID)).bold("]").space().bold(report.getString(ReportData.TITLE)).endl(2);

        append("Assigned To: ").append(report.getString(ReportData.ASSIGNEDTO)).endl();
        append("Severity: ").append(report.getString(ReportData.SEVERITY)).endl();
        append("Priority: ").append(report.getString(ReportData.PRIORITY)).endl();
        append("Target Milestone: ").append(report.getString(ReportData.TARGETMILESTONE)).endl();
        endl();
        append("Operating System: ").append(report.getString(ReportData.OPERATINGSYSTEM)).endl();
        append("Platform: ").append(report.getString(ReportData.PLATFORM)).endl();
        
        endl();
        
        /* append first comment
        final Iterator iter= bugzillaReport.getComments().iterator();
        if (iter.hasNext()) {
            append((BugzillaComment)iter.next());
        }
        */
        return this;
    }

    private BugzillaModelPresenter append(String suffix) {
        fBuffer.append(suffix);
        return this;
    }
    
    private BugzillaModelPresenter bold(String suffix) {
        final int offset= fBuffer.length();
        final int length= suffix.length();
        fBuffer.append(suffix);
        fPresentation.addStyleRange(new StyleRange(offset, length, null, null, SWT.BOLD));
        return this;
    }
    
    private BugzillaModelPresenter italic(String suffix) {
        final int offset= fBuffer.length();
        final int length= suffix.length();
        fBuffer.append(suffix);
        fPresentation.addStyleRange(new StyleRange(offset, length, null, null, SWT.ITALIC));
        return this;
    }
    
    private BugzillaModelPresenter endl() {
        fBuffer.append("\n");
        return this;
    }
    
    private BugzillaModelPresenter endl(int n) {
        for (int i= 0; i < n; ++i)
            fBuffer.append("\n");
        return this;
    }
    
    private BugzillaModelPresenter space() {
        fBuffer.append(" ");
        return this;
    }
    
//    private BugzillaModelPresenter space(int n) {
//        for (int i= 0; i < n; i++)
//            fBuffer.append(" ");
//        return this;
//    }
    
    private BugzillaModelPresenter append(Date date) {
        fBuffer.append(DateFormat.getDateTimeInstance().format(date));
        return this;
    }

}
