/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.view;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.eclipse.team.bugzilla.model.IRemoteModel;
import org.eclipse.team.bugzilla.operations.RefreshOperation;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.core.BackgroundEventHandler;

/**
 * This class will do a delayed refesh of a selected element if
 * it is being presented. It uses a delay to prevent refreshes
 * to happen when the selection is changing quickly (for example,
 * due to navigation using arrow keys).
 */
class DelayedRefreshPresenter extends BackgroundEventHandler {

    private static final int FETCH_NEEDED = 1;
    private static final int ABORT_FETCH = 2;
    private static final long DELAY = 500; // half a second

    PresentationEvent currentSelection;
    
    private static class PresentationEvent extends Event {
        private final IRemoteModel element;
        private long selectionTime;
        public PresentationEvent(IRemoteModel element) {
            super(element == null ? ABORT_FETCH : FETCH_NEEDED);
            this.element = element;
            if (element != null)
                this.selectionTime = System.currentTimeMillis();
        }
        public IRemoteModel getElement() {
            return element;
        }
        public long getSelectionTime() {
            return selectionTime;
        }
    }
    
    public DelayedRefreshPresenter() {
        super("Background refresh", "An error occurred while refreshing");
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.internal.core.BackgroundEventHandler#doDispatchEvents(org.eclipse.core.runtime.IProgressMonitor)
     */
    protected boolean doDispatchEvents(IProgressMonitor monitor) throws TeamException {
        if (isQueueEmpty() && currentSelection != null) {
            try {
                new RefreshOperation(null, new IRemoteModel[] { currentSelection.getElement() }).run(monitor);
            } catch (InvocationTargetException e) {
                //TODO: We're a little bit verbose at the moment. Do not log standard stuff, but give visual indication.
                BugzillaPlugin.log((e.getCause() instanceof BugzillaException) ? (BugzillaException)e.getCause() : new BugzillaException("Error during background refresh", e.getCause()));
            } catch (InterruptedException e) {
                // Ignore
            }
            return true;
        }
        return false;
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.internal.core.BackgroundEventHandler#processEvent(org.eclipse.team.internal.core.BackgroundEventHandler.Event, org.eclipse.core.runtime.IProgressMonitor)
     */
    protected void processEvent(Event event, IProgressMonitor monitor) throws CoreException {
        switch (event.getType()) {
        	case FETCH_NEEDED:
        	    currentSelection = (PresentationEvent)event;
        	    break;
        	case ABORT_FETCH:
        	    currentSelection = null;
        	    break;
        }
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.internal.core.BackgroundEventHandler#getShortDispatchDelay()
     */
    protected long getShortDispatchDelay() {
        return DELAY;
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.internal.core.BackgroundEventHandler#getLongDispatchDelay()
     */
    protected long getLongDispatchDelay() {
        return getShortDispatchDelay();
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.internal.core.BackgroundEventHandler#isReadyForDispatch(boolean)
     */
    protected boolean isReadyForDispatch(boolean wait) {
        if (super.isReadyForDispatch(wait)) {
            if (currentSelection != null) {
	            long currentTime = System.currentTimeMillis();
	            long diff = currentTime - currentSelection.getSelectionTime();
	            if (diff < DELAY) {
	                // TODO: Tried to wait but got an Illegal Monitor state return false;
	            }
	            // We are ready to dispatch if the queue is empty
	            return isQueueEmpty();
            }
            return true;
        }
        return false;
    }

    /**
     * Refresh the given model element if it remains selected
     * @param model the model element to refresh or <code>null</code>
     * if the selection does not need refreshing
     * 
     * @return <code>true</code> if the model gets queued for a refresh, <code>false</code> otherwise.
     */
    public boolean refresh(BugzillaModel model) {
        if (model instanceof IRemoteModel && ((IRemoteModel)model).needsRefresh() || model == null) {
            queueEvent(new PresentationEvent((IRemoteModel)model), false /* add to end of queue */);
            return true;
        }
        return false;
    }
}
