/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.xml;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.datatypes.QueryData;
import org.eclipse.team.bugzilla.datatypes.ReportData;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * SAX parser handler for query data
 */
public class QueryHandler extends DefaultHandler  {
    
    Stack elementStack = new Stack();

    public static final String RDF_TAG = "RDF"; //$NON-NLS-1$
    public static final String RESULT_TAG = "bz:result"; //$NON-NLS-1$
    public static final String BUG_LIST_TAG = "bz:bugs"; //$NON-NLS-1$
    public static final String BUG_TAG = "bz:bug"; //$NON-NLS-1$
    public static final String BUG_ID_TAG = "bz:id"; //$NON-NLS-1$
    public static final String BUG_SUMMARY_TAG = "bz:summary"; //$NON-NLS-1$
    public static final String BUG_OPEN_DATE_TAG = "bz:opendate"; //$NON-NLS-1$
    public static final String BUG_SEVERITY_TAG = "bz:severity"; //$NON-NLS-1$
    public static final String BUG_PRIORITY_TAG = "bz:priority"; //$NON-NLS-1$
    public static final String BUG_OWNER_TAG = "bz:owner"; //$NON-NLS-1$
    public static final String BUG_STATUS_TAG = "bz:status"; //$NON-NLS-1$
    public static final String BUG_TARGET_MILESTONE_TAG = "bz:target_milestone"; //$NON-NLS-1$
    
    /*
     * Field that appears in bz:result and bz:bug tag
     */
    public static final String ABOUT_FIELD = "about"; //$NON-NLS-1$
    
    /*
     * Fields that appear in the RDF tag to identify the namesapces in use
     */
    public static final String XMLNS_FIELD = "xmlns"; //$NON-NLS-1$
    public static final String BUG_XMLNS_FIELD = "xmlns:bz"; //$NON-NLS-1$
    
    public static final String EXPECTED_XMLNS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"; //$NON-NLS-1$
    public static final String EXPECTED_BUG_XMLNS = "http://www.bugzilla.org/rdf#"; //$NON-NLS-1$

    public static final Map tag2data = new HashMap();
    
    static {
        tag2data.put(BUG_ID_TAG, ReportData.ID);
        tag2data.put(BUG_SUMMARY_TAG, ReportData.TITLE);
        //tag2data.put(BUG_OPEN_DATE_TAG, ReportData.CREATION_TIMESTAMP);
        tag2data.put(BUG_SEVERITY_TAG, ReportData.SEVERITY);
        tag2data.put(BUG_PRIORITY_TAG, ReportData.PRIORITY);
        tag2data.put(BUG_OWNER_TAG, ReportData.ASSIGNEDTO);
        tag2data.put(BUG_STATUS_TAG, ReportData.STATUS);
        tag2data.put(BUG_TARGET_MILESTONE_TAG, ReportData.TARGETMILESTONE);
        // TODO: There may be more
        
        /* FIXME: 2.18 fix */
        tag2data.put("bz:bug_severity", ReportData.SEVERITY);
        tag2data.put("bz:rep_platform", ReportData.PLATFORM);
        tag2data.put("bz:bug_status", ReportData.STATUS);
        tag2data.put("bz:short_short_desc", ReportData.TITLE);
    }

    private ReportData currentBugData;
    
    private QueryData data = new QueryData();
    
    private StringBuffer textBuffer = new StringBuffer();
    
    /**
     * Return the query data that was fetched.
     * @return the query data that was fetched
     */
    public QueryData getQueryData() {
        return data;
    }

	private String getElementName(String localName, String qName) {
		if (localName != null && localName.length() > 0) 
			return localName;
		return qName;
	}
	
    private void addError(Status status) {
        data.addError(status);
    }
    
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        
        String name = getElementName(localName, qName);
        elementStack.push(name);
        if (name.equals(RDF_TAG)) {
            validateNamespaces(attributes);
        } else if (name.equals(BUG_TAG)) {
            currentBugData = new ReportData();
        } else if (isBugDataField(name)) {
            clearText();
        }
    }

    private boolean isBugDataField(String name) {
        return tag2data.containsKey(name);
    }

    private String getDataKey(String name) {
        return (String)tag2data.get(name);
    }

    
    private void validateNamespaces(Attributes attributes) {
        String ns = attributes.getValue(XMLNS_FIELD);
        if (!ns.equals(EXPECTED_XMLNS)) {
            addError(new Status(IStatus.ERROR, BugzillaPlugin.ID, 0, "Expected xml namespace " + EXPECTED_XMLNS + " but was " + ns, null));
        }
        String bns = attributes.getValue(BUG_XMLNS_FIELD);
        if (!bns.equals(EXPECTED_BUG_XMLNS)) {
            addError(new Status(IStatus.ERROR, BugzillaPlugin.ID, 0, "Expected bug namespace " + EXPECTED_BUG_XMLNS + " but was " + bns, null));
        }
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        
        String name = getElementName(localName, qName);
        if (elementStack.peek().equals(name)) {
            elementStack.pop();
        } else {
            addError(new Status(IStatus.ERROR, BugzillaPlugin.ID, 0, "Expected tag " + elementStack.peek()+ " but was " + name, null));
        }
        
        if (name.equals(BUG_TAG)) {
            if (currentBugData != null && currentBugData.getString(ReportData.ID) != null) {
                data.addBug(currentBugData);
                currentBugData = null;
            }
        } else if (isBugDataField(name)) {
            if (currentBugData != null) {
	            String text = getText();
                currentBugData.set(getDataKey(name), text);
            }
        }
    }

    private void clearText() {
        textBuffer.setLength(0);
    }
    
    private String getText() {
        return textBuffer.toString();
    }
    
    public void characters(char[] ch, int start, int length) throws SAXException {
        textBuffer.append(ch, start, length);
    }
}
